@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Variables that are needed globally, and even by the very basic libs.
  *
@@ -400,6 +400,10 @@ after_dot:
 					{
 						requested_precision = src_width;
 					}
+					if (requested_precision > width)
+					{
+						requested_precision = width;
+					}
 					if (requested_width==INT_MAX)
 					{
 						requested_width = requested_precision;
@@ -536,6 +540,7 @@ after_dot:
 										memmove (buffer + 1, buffer, strlen (buffer) + 1);
 										buffer[0] = '0';
 									}
+									requested_precision++;
 								}
 								if ((requested_width > requested_precision) && minusflag)
 								{
@@ -709,6 +714,8 @@ struct console_t Console =
 #endif
 	try_open_jpeg,
 	try_open_png,
+	EditStringUTF8z,
+	EditStringUTF8,
 	80,      /* TextHeight */
 	25,      /* TextWidth */
 	0,       /* TextGUIOverlay */
