@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * OCP.INI file and environment reading functions
  *
@@ -46,12 +46,17 @@
 #include "psetting.h"
 #include "stuff/compat.h"
 
-char *cfProgramDir;
-char *cfProgramDirAutoload;
+char *cfProgramPath;
+char *cfProgramPathAutoload;
 
 #define KEYBUF_LEN 105
 #define STRBUF_LEN 405
 #define COMMENTBUF_LEN 256
+#define COMMENT_INDENT 26
+
+#ifndef MAX
+# define MAX(x,y) ((x)>(y)?(x):(y))
+#endif
 
 struct profilekey
 {
@@ -159,7 +164,8 @@ static int cfReadINIFile(int argc, char *argv[])
 	char linebuffer[1024];
 	/*  int curapp=-1;*/
 
-	makepath_malloc (&path, 0, cfConfigDir, "ocp.ini", 0);
+	path = malloc (strlen (configAPI.ConfigHomePath) + strlen ("ocp.ini") + 1);
+	sprintf (path, "%socp.ini", configAPI.ConfigHomePath);
 
 	strcpy(keybuf, "");
 
@@ -270,7 +276,7 @@ static int cfReadINIFile(int argc, char *argv[])
 				argvstat=argv[c]+2;
 				while (*argvstat)
 				{
-					char *temp=index(argvstat, ',');
+					char *temp=strchr(argvstat, ',');
 					int index=cfINIApps[cfINInApps-1].nkeys;
 
 					if (!temp)
@@ -463,6 +469,7 @@ static void _cfSetProfileString(const char *app, const char *key, const char *st
 				if (cfINIApps[i].keys[j].key)
 					if (!strcasecmp(cfINIApps[i].keys[j].key, key))
 					{
+						if (cfINIApps[i].keys[j].str == str) return;
 						free(cfINIApps[i].keys[j].str);
 						cfINIApps[i].keys[j].str = strdup (str);
 						return;
@@ -538,6 +545,42 @@ static void _cfSetProfileBool(const char *app, const char *key, const int str)
 	_cfSetProfileString(app, key, (str?"on":"off"));
 }
 
+static const char *_cfGetProfileComment(const char *app, const char *key, const char *def)
+{
+	int i,j;
+	for (i=0; i<cfINInApps; i++)
+		if (!strcasecmp(cfINIApps[i].app, app))
+			for (j=0; j<cfINIApps[i].nkeys; j++)
+				if (cfINIApps[i].keys[j].key)
+					if (!strcasecmp(cfINIApps[i].keys[j].key, key))
+						return cfINIApps[i].keys[j].comment ? cfINIApps[i].keys[j].comment : def;
+	return def;
+}
+
+static void _cfSetProfileComment(const char *app, const char *key, const char *comment)
+{
+	int i, j;
+	for (i=0; i<cfINInApps; i++)
+	{
+		if (!strcasecmp(cfINIApps[i].app, app))
+		{
+			for (j=0; j<cfINIApps[i].nkeys; j++)
+			{
+				if (cfINIApps[i].keys[j].key)
+				{
+					if (!strcasecmp(cfINIApps[i].keys[j].key, key))
+					{
+						if (cfINIApps[i].keys[j].comment == comment) return;
+						free(cfINIApps[i].keys[j].comment);
+						cfINIApps[i].keys[j].comment = strdup (comment);
+						return;
+					}
+				}
+			}
+		}
+	}
+}
+
 static void _cfRemoveEntry(const char *app, const char *key)
 {
 	int i, j;
@@ -596,7 +639,7 @@ static void _cfRemoveProfile(const char *app)
 	}
 }
 
-int cfCountSpaceList(const char *str, int maxlen)
+static int _cfCountSpaceList(const char *str, int maxlen)
 {
 	int i=0;
 	while (1)
@@ -615,7 +658,7 @@ int cfCountSpaceList(const char *str, int maxlen)
 	}
 }
 
-int cfGetSpaceListEntry(char *buf, const char **str, int maxlen)
+static int _cfGetSpaceListEntry(char *buf, const char **str, int maxlen)
 {
 	while (1)
 	{
@@ -644,28 +687,28 @@ int cfGetConfig(int argc, char *argv[])
 		return -1; /* no config at all pigs! */
 	if (cfReadINIFile(argc, argv))
 	{
-		fprintf(stderr, "Failed to read ocp.ini\nPlease put it in ~/.ocp/\n");
+		fprintf(stderr, "Failed to read ocp.ini\nPlease put it in ~/.ocp/ or $XDG_CONFIG_HOME/ocp/\n");
 		return -1;
 	}
 
 	t=_cfGetProfileString("general", "datadir", NULL);
 	if (t)
 	{
-		free (cfDataDir);
-		cfDataDir = strdup (t);
+		free (cfDataPath);
+		cfDataPath = strdup (t);
 	}
 
 	if ((t=_cfGetProfileString("general", "tempdir", t)))
 	{
-		cfTempDir = strdup (t);
+		cfTempPath = strdup (t);
 	} else if ((t=getenv("TEMP")))
 	{
-		cfTempDir = strdup (t);
+		cfTempPath = strdup (t);
 	} else if ((t=getenv("TMP")))
 	{
-		cfTempDir = strdup (t);
+		cfTempPath = strdup (t);
 	} else {
-		cfTempDir = strdup ("/tmp/");
+		cfTempPath = strdup ("/tmp/");
 	}
 
 #ifdef PSETTING_DEBUG
@@ -716,9 +759,9 @@ static int _cfStoreConfig(void)
 	char *path;
 	FILE *f;
 	int i, j;
-	char buffer[2+KEYBUF_LEN+1+STRBUF_LEN+COMMENTBUF_LEN+32+1+1];
 
-	makepath_malloc (&path, 0, cfConfigDir, "ocp.ini", 0);
+	path = malloc (strlen (configAPI.ConfigHomePath) + strlen ("ocp.ini") + 1);
+	sprintf (path, "%socp.ini", configAPI.ConfigHomePath);
 
 	if (!(f=fopen(path, "w")))
 	{
@@ -729,42 +772,46 @@ static int _cfStoreConfig(void)
 	free (path); path=0;
 
 	for (i=0;i<cfINInApps;i++)
+	{
 		if (cfINIApps[i].linenum>=0)
 		{
-			strcpy(buffer, "[");
-			strcat(buffer, cfINIApps[i].app);
-			strcat(buffer, "]");
+			if (i)
+			{
+				fprintf (f, "\n");
+			}
+			fprintf (f, "[%.*s]", KEYBUF_LEN, cfINIApps[i].app);
 			if (cfINIApps[i].comment)
 			{
-				int n=strlen(buffer)-32;
-				if (n>0)
-					strncat(buffer, "                                ", n);
-				strcat(buffer, cfINIApps[i].comment);
+				fprintf (f, "%*s%.*s",
+			                MAX (0, COMMENT_INDENT - 2 - (int)strlen(cfINIApps[i].app)), "",
+					COMMENTBUF_LEN, cfINIApps[i].comment);
 			}
-			strcat(buffer, "\n");
+			fprintf (f, "\n");
 
-			fprintf(f, "%s", buffer);
 			for (j=0;j<cfINIApps[i].nkeys;j++)
+			{
 				if (cfINIApps[i].keys[j].linenum>=0)
 				{
 					if (cfINIApps[i].keys[j].key)
 					{
-						strcpy(buffer, "  ");
-						strcat(buffer, cfINIApps[i].keys[j].key);
-						strcat(buffer, "=");
-						strcat(buffer, cfINIApps[i].keys[j].str);
+						fprintf (f, "  %.*s=%.*s",
+							KEYBUF_LEN, cfINIApps[i].keys[j].key,
+							STRBUF_LEN, cfINIApps[i].keys[j].str);
 						if (cfINIApps[i].keys[j].comment)
 						{
-							while (strlen(buffer)<32)
-								strcat(buffer, " ");
-							strcat(buffer, cfINIApps[i].keys[j].comment);
+							fprintf (f, "%*s%.*s",
+								MAX (0, COMMENT_INDENT - 3 - (int)strlen(cfINIApps[i].keys[j].key) - (int)strlen(cfINIApps[i].keys[j].str)), "",
+								COMMENTBUF_LEN, cfINIApps[i].keys[j].comment);
 						}
-					} else
-						strcpy(buffer, cfINIApps[i].keys[j].comment);
-					strcat(buffer, "\n");
-					fprintf(f, "%s", buffer);
+						fprintf (f, "\n");
+					} else if (cfINIApps[i].keys[j].comment)
+					{
+						fprintf (f, "%.*s\n", COMMENTBUF_LEN, cfINIApps[i].keys[j].comment);
+					}
 				}
+			}
 		}
+	}
 	fclose(f);
 	return 0;
 }
@@ -781,6 +828,23 @@ struct configAPI_t configAPI =
 	_cfGetProfileInt,
 	_cfGetProfileInt2,
 	_cfSetProfileInt,
+	_cfGetProfileComment,
+	_cfSetProfileComment,
 	_cfRemoveEntry,
-	_cfRemoveProfile
+	_cfRemoveProfile,
+	0,           /*       HomeDir */
+	0,           /* ConfigHomeDir */
+	0,           /*   DataHomeDir */
+	0,           /*       DataDir */
+	0,           /*       TempDir */
+	0,           /*       HomePath */
+	0,           /* ConfigHomePath */
+	0,           /*   DataHomePath */
+	0,           /*       DataPath */
+	0,           /*       TempPath */
+	0,           /* ConfigSec */
+	0,           /*  SoundSec */
+	0,           /* ScreenSec */
+	_cfCountSpaceList,
+	_cfGetSpaceListEntry
 };
