@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Wavetable Device: Software Mixer for sample stream output via devp
  *
@@ -40,13 +40,15 @@
 #include <unistd.h>
 #include "types.h"
 #include "boot/plinkman.h"
+#include "boot/psetting.h"
 #include "cpiface/cpiface.h"
-#include "dev/imsdev.h"
+#include "dev/deviwave.h"
 #include "dev/mcp.h"
 #include "dev/mix.h"
 #include "dev/player.h"
 #include "dev/plrasm.h"
-#include "devwmix.h"
+#include "dev/postproc.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
 #include "dwmix.h"
 #include "dwmixa.h"
@@ -55,9 +57,14 @@
 #define MIXBUFLEN 4096
 #define MAXCHAN 255
 
-extern struct sounddevice mcpMixer;
+static const struct mcpDriver_t mcpMixer;
+static const struct mcpDriver_t mcpMixerQ;
 
-static struct mixqpostprocregstruct *postprocs;
+static const struct mixAPI_t *mix;
+
+#define MIX_MAX_POSTPROC 10
+static const struct PostProcIntegerRegStruct *postproc[MIX_MAX_POSTPROC];
+static int                                    postprocs;
 
 static int quality;
 static int resample;
@@ -108,9 +115,12 @@ static int masterbal;
 static int masterpan;
 static int mastersrnd;
 static int masterrvb;
+static int masterchr;
 
 static uint32_t IdleCache; /* To prevent devpDisk lockup */
 
+static int devwMixProcKey (uint16_t key);
+
 static void calcinterpoltabr(void)
 	/* used by OpenPlayer */
 {
@@ -330,9 +340,9 @@ static void fadechanq(int *fade, struct channel *c)
 {
 	int s;
 	if (c->status&MIXRQ_PLAY16BIT)
-		s=((short*)((unsigned long)c->samp*2))[c->pos];
+		s=c->realsamp.bit16[c->pos];
 	else
-		s=(((signed char*)c->samp)[c->pos])<<8;
+		s=(c->realsamp.bit8[c->pos])<<8;
 	fade[0]+=(c->curvols[0]*s)>>8;
 	fade[1]+=(c->curvols[1]*s)>>8;
 	c->curvols[0]=c->curvols[1]=0;
@@ -397,7 +407,6 @@ static void devwMixIdle  (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	/* mixer */
 	int i;
-	struct mixqpostprocregstruct *mode;
 
 	if (!channelnum)
 		return;
@@ -414,15 +423,15 @@ static void devwMixIdle  (struct cpifaceSessionAPI_t *cpifaceSession)
 
 	if (_pause)
 	{
-		plrDevAPI->Pause (1);
+		cpifaceSession->plrDevAPI->Pause (1);
 
 	} else {
 		void *targetbuf;
 		unsigned int targetlength; /* in samples */
 
-		plrDevAPI->Pause (0);
+		cpifaceSession->plrDevAPI->Pause (0);
 
-		plrDevAPI->GetBuffer (&targetbuf, &targetlength);
+		cpifaceSession->plrDevAPI->GetBuffer (&targetbuf, &targetlength);
 
 		while (targetlength)
 		{
@@ -445,8 +454,10 @@ static void devwMixIdle  (struct cpifaceSessionAPI_t *cpifaceSession)
 					playchannelq(i, targetlength);
 			}
 
-			for (mode=postprocs; mode; mode=mode->next)
-				mode->Process(buf32, targetlength, samprate);
+			for (i=0; i < postprocs; i++)
+			{
+				postproc[i]->Process (cpifaceSession, buf32, targetlength, samprate);
+			}
 
 			mixrClip((char*)targetbuf, buf32, targetlength << 1 /* stereo */, amptab, clipmax);
 
@@ -462,9 +473,9 @@ static void devwMixIdle  (struct cpifaceSessionAPI_t *cpifaceSession)
 
 			playsamps+=targetlength;
 
-			plrDevAPI->CommitBuffer (targetlength);
+			cpifaceSession->plrDevAPI->CommitBuffer (targetlength);
 
-			plrDevAPI->GetBuffer (&targetbuf, &targetlength);
+			cpifaceSession->plrDevAPI->GetBuffer (&targetbuf, &targetlength);
 
 			if (_pause)
 			{
@@ -473,41 +484,19 @@ static void devwMixIdle  (struct cpifaceSessionAPI_t *cpifaceSession)
 		}
 	}
 
-	plrDevAPI->Idle();
+	cpifaceSession->plrDevAPI->Idle();
 
 	BARRIER
 
 	clipbusy--;
 }
 
-#ifdef __STRANGE_BUG__
-void *GetReturnAddress12();
-#pragma aux GetReturnAddress12="mov eax, [esp+12]" value [eax];
-
-void sbDumpRetAddr(void *ptr)
-{
-  linkaddressinfostruct a;
-  lnkGetAddressInfo(a, ptr);
-  printf("--> called from %s.%s+0x%x, %s.%d+0x%x\n", a.module, a.sym,
-	 a.symoff, a.source, a.line, a.lineoff);
-  fflush(stdout);
-}
-#endif
-
-static void devwMixSET(int ch, int opt, int val)
+static void devwMixSET (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
 {
 	/* Refered by OpenPlayer
 	 */
 	struct channel *chn;
 
-#ifdef __STRANGE_BUG__
-  if ((opt==mcpMasterBalance)&&(val<-64 || val>64))  /* got that damn bug. */
-    {
-      sbDumpRetAddr(GetReturnAddress12());
-      printf("mcpSet(%d, %d, %d);\n", ch, opt, val);
-    }
-#endif
-
 	if (ch>=channelnum)
 		ch=channelnum-1;
 	if (ch<0)
@@ -545,12 +534,10 @@ static void devwMixSET(int ch, int opt, int val)
 			if (chn->samptype&mcpSamp16Bit)
 			{
 				chn->status|=MIXRQ_PLAY16BIT;
-				chn->samp=(void*)((unsigned long)chn->samp>>1);
 			}
 			if (chn->samptype&mcpSampStereo)
 			{
 				chn->status|=MIXRQ_PLAYSTEREO;
-				chn->samp=(void*)((unsigned long)chn->samp>>1);
 			}
 			if (chn->samptype&mcpSampSLoop)
 			{
@@ -657,7 +644,7 @@ static void devwMixSET(int ch, int opt, int val)
 			}
 		case mcpCPitch:
 			chn->orgfrq=8363;
-			chn->orgdiv=mcpGetFreq8363(-val);
+			chn->orgdiv=cpifaceSession->mcpAPI->GetFreq8363(-val);
 			calcstep(chn);
 			break;
 		case mcpCPitchFix:
@@ -683,9 +670,7 @@ static void devwMixSET(int ch, int opt, int val)
 			break;
 
 		case mcpMasterPanning:
-#ifndef __STRANGE_BUG__
 			if (val>=-64 && val<=64)
-#endif
 				masterpan=(val<-64)?-64:(val>64)?64:val;
 			calcvols();
 			break;
@@ -698,8 +683,11 @@ static void devwMixSET(int ch, int opt, int val)
 			mastersrnd=val?1:0;
 			calcvols();
 			break;
+		case mcpMasterChorus:
+			masterchr=(val>=64)?64:(val<0)?0:val;
+			break;
 		case mcpMasterReverb:
-			masterrvb=(val>=64)?63:(val<-64)?-64:val;
+			masterrvb=(val>=64)?64:(val<0)?0:val;
 			break;
 		case mcpMasterSpeed:
 			relspeed=(val<16)?16:val;
@@ -717,7 +705,7 @@ static void devwMixSET(int ch, int opt, int val)
 			if (channelnum)
 			{
 				calcamptab(amplify);
-				mixSetAmplify(amplify);
+				mix->mixSetAmplify (cpifaceSession, amplify);
 			}
 			break;
 		case mcpMasterPause:
@@ -727,7 +715,7 @@ static void devwMixSET(int ch, int opt, int val)
 	}
 }
 
-static int devwMixGET(int ch, int opt)
+static int devwMixGET (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt)
 {
 	/* Refered by OpenPlayer
 	 */
@@ -750,20 +738,30 @@ static int devwMixGET(int ch, int opt)
 			return umuldiv(cmdtimerpos, 256, samprate);
 		case mcpMasterReverb:
 			return masterrvb;
+		case mcpMasterChorus:
+			return masterchr;
 	}
 	return 0;
 }
 
+static void devwMixGetVolRegs (struct cpifaceSessionAPI_t *cpifaceSession, void (*Callback)(struct cpifaceSessionAPI_t *cpifaceSession, const struct ocpvolregstruct *x))
+{
+	int i;
+	for (i=0; i < postprocs; i++)
+	{
+		if (postproc[i]->VolRegs)
+		{
+			Callback (cpifaceSession, postproc[i]->VolRegs);
+		}
+	}
+}
+
 static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate)
 	/* Refered to by OpenPlayer to mixInit */
 {
 	struct channel *c=&channels[ch];
 
-	if (c->status&MIXRQ_PLAY16BIT)
-		chn->samp=(void*)((unsigned long)c->samp<<1);
-	else
-		chn->samp=c->samp;
-	chn->realsamp.fmt=chn->samp; /* at this point, they match */
+	chn->realsamp.fmt=c->samp;
 	chn->length=c->length;
 	chn->loopstart=c->loopstart;
 	chn->loopend=c->loopend;
@@ -787,11 +785,11 @@ static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate
 		chn->status|=MIX_INTERPOLATE;
 }
 
-static int devwMixLoadSamples(struct sampleinfo *sil, int n)
+static int devwMixLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct sampleinfo *sil, int n)
 {
 #if 0
 	int i;
-	if (!mcpReduceSamples(sil, n, 0x40000000, mcpRedToMono))
+	if (!cpifaceSession->mcpAPI->ReduceSamples(sil, n, 0x40000000, mcpRedToMono))
 		return 0;
 
 	samples=malloc(sizeof(*sil)*n);
@@ -807,7 +805,7 @@ static int devwMixLoadSamples(struct sampleinfo *sil, int n)
 	samplenum=n;
 
 #else
-	if (!mcpReduceSamples(sil, n, 0x40000000, mcpRedToMono))
+	if (!cpifaceSession->mcpAPI->ReduceSamples(sil, n, 0x40000000, mcpRedToMono))
 		return 0;
 
 	samples=sil;
@@ -819,8 +817,8 @@ static int devwMixLoadSamples(struct sampleinfo *sil, int n)
 static int devwMixOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t *cpifaceSession), struct ocpfilehandle_t *source_file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	uint32_t currentrate;
-	struct mixqpostprocregstruct *mode;
 	enum plrRequestFormat format;
+	int i;
 
 	fadedown[0]=fadedown[1]=0;
 	IdleCache=playsamps=0;
@@ -829,7 +827,7 @@ static int devwMixOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t *
 		chan=MAXCHAN;
 	}
 
-	if (!plrDevAPI)
+	if (!cpifaceSession->plrDevAPI)
 	{
 		return 0;
 	}
@@ -885,15 +883,15 @@ static int devwMixOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t *
 		goto error_out;
 	}
 
-	currentrate=mcpMixProcRate/chan;
-	samprate=(currentrate>mcpMixMaxRate)?mcpMixMaxRate:currentrate;
+	currentrate = cpifaceSession->mcpAPI->MixProcRate / chan;
+	samprate = (currentrate > cpifaceSession->mcpAPI->MixMaxRate) ? cpifaceSession->mcpAPI->MixMaxRate : currentrate;
 	format=PLR_STEREO_16BIT_SIGNED;
-	if (!plrDevAPI->Play (&samprate, &format, source_file, cpifaceSession))
+	if (!cpifaceSession->plrDevAPI->Play (&samprate, &format, source_file, cpifaceSession))
 	{
 		goto error_out;
 	}
 
-	if (!mixInit(GetMixChannel, resample, chan, amplify, cpifaceSession))
+	if (!mix->mixInit (cpifaceSession, GetMixChannel, resample, chan, amplify))
 	{
 		goto error_out_plrDevAPI_Play;
 	}
@@ -917,6 +915,8 @@ static int devwMixOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t *
 	channelnum=chan;
 	cpifaceSession->PhysicalChannelCount = chan;
 	cpifaceSession->mcpGet = devwMixGET;
+	cpifaceSession->mcpGetVolRegs = devwMixGetVolRegs;
+
 	cpifaceSession->mcpSet = devwMixSET;
 
 	calcamptab(amplify);
@@ -926,15 +926,18 @@ static int devwMixOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t *
 	tickplayed=0;
 	cmdtimerpos=0;
 
-	for (mode=postprocs; mode; mode=mode->next)
-		if (mode->Init)
-			mode->Init(samprate);
+	for (i=0; i < postprocs; i++)
+	{
+		postproc[i]->Init(samprate);
+	}
+
+	cpifaceSession->mcpActive = 1;
 
 	return 1;
 
-	// mixClose();
+	// mix->mixClose (cpifaceSession);
 error_out_plrDevAPI_Play:
-	plrDevAPI->Stop();
+	cpifaceSession->plrDevAPI->Stop (cpifaceSession);
 error_out:
 	free (amptab);        amptab = 0;
 	free (voltabsr);      voltabsr = 0;
@@ -949,19 +952,23 @@ error_out:
 	return 0;
 }
 
-static void devwMixClosePlayer()
+static void devwMixClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	struct mixqpostprocregstruct *mode;
+	int i;
 
-	plrDevAPI->Stop();
+	if (cpifaceSession->plrDevAPI)
+	{
+		cpifaceSession->plrDevAPI->Stop (cpifaceSession);
+	}
 
 	channelnum=0;
 
-	mixClose();
+	mix->mixClose (cpifaceSession);
 
-	for (mode=postprocs; mode; mode=mode->next)
-		if (mode->Close)
-			mode->Close();
+	for (i=0; i < postprocs; i++)
+	{
+		postproc[i]->Close();
+	}
 
 	if (voltabsr) free(voltabsr);
 	if (interpoltabr) free(interpoltabr);
@@ -981,6 +988,8 @@ static void devwMixClosePlayer()
 	voltabsq=NULL;
 	interpoltabq=NULL;
 	interpoltabq2=NULL;
+
+	cpifaceSession->mcpActive = 0;
 }
 
 static const struct mcpDevAPI_t devwMix =
@@ -988,13 +997,26 @@ static const struct mcpDevAPI_t devwMix =
 	devwMixOpenPlayer,
 	devwMixLoadSamples,
 	devwMixIdle,
-	devwMixClosePlayer
+	devwMixClosePlayer,
+	devwMixProcKey
 };
 
-static int wmixInit(const struct deviceinfo *dev)
+static void mixrRegisterPostProc (const struct PostProcIntegerRegStruct *post)
 {
-	resample=!!(dev->opt&MIXRQ_RESAMPLE);
-	quality=!!dev->subtype;
+	if (postprocs >= MIX_MAX_POSTPROC)
+	{
+		return;
+	}
+	postproc[postprocs] = post;
+	postprocs++;
+}
+
+static const struct mcpDevAPI_t *wmixInit (const struct mcpDriver_t *driver, const struct configAPI_t *config, const struct mixAPI_t *mixAPI)
+{
+	char regname[50];
+	const char *regs;
+
+	mix = mixAPI;
 
 	amplify=65535;
 	relspeed=256;
@@ -1006,114 +1028,80 @@ static int wmixInit(const struct deviceinfo *dev)
 	mastersrnd=0;
 	channelnum=0;
 
-	mcpDevAPI = &devwMix;
+	quality = (driver == &mcpMixerQ);
+	resample = config->GetProfileBool(driver->name, "mixresample", 0, 0);
 
-	return 1;
-}
+	fprintf(stderr, "[%s] %s C version (resample=%d)\n", driver->name, quality?"dwmixaq.c":"dwmixa.c", resample);
 
-static void wmixClose(void)
-{
-	if (mcpDevAPI == &devwMix)
+	regs = config->GetProfileString(driver->name, "postprocs", "");
+	while (config->GetSpaceListEntry(regname, &regs, 49))
 	{
-		mcpDevAPI = 0;
-	}
-}
-
-
-static int wmixDetect(struct deviceinfo *c)
-{
-	c->devtype=&mcpMixer;
-	c->port=-1;
-	c->port2=-1;
-/*
-	c->irq=-1;
-	c->irq2=-1;
-	c->dma=-1;
-	c->dma2=-1;
-*/
-	if (c->subtype==-1)
-		c->subtype=0;
-	c->chan=/*(MAXCHAN>99)?99:MAXCHAN;*/MAXCHAN;
-	c->mem=0;
-	return 1;
-}
-
-void mixrRegisterPostProc(struct mixqpostprocregstruct *mode)
-{
-	mode->next=postprocs;
-	postprocs=mode;
-}
-
-#include "dev/devigen.h"
-#include "boot/psetting.h"
-#include "dev/deviplay.h"
-#include "boot/plinkman.h"
-
-static struct mixqpostprocaddregstruct *postprocadds;
-
-static uint32_t mixGetOpt(const char *sec)
-{
-	uint32_t opt=0;
-	if (cfGetProfileBool(sec, "mixresample", 0, 0))
-		opt|=MIXRQ_RESAMPLE;
-	return opt;
-}
-
-static void mixrInit(const char *sec)
-{
-	char regname[50];
-	const char *regs;
-
-
-	fprintf(stderr, "[devwmix] INIT, ");
-	if (!quality)
-	{
-		fprintf(stderr, "using dwmixa.c C version\n");
-	} else {
-		fprintf(stderr, "using dwmixaq.c C version\n");
-	}
-
-	postprocs=0;
-	regs=cfGetProfileString(sec, "postprocs", "");
-
-	while (cfGetSpaceListEntry(regname, &regs, 49))
-	{
-		void *reg=_lnkGetSymbol(regname);
-		fprintf(stderr, "[%s] registering %s: %p\n", sec, regname, reg);
-		if (reg)
-			mixrRegisterPostProc((struct mixqpostprocregstruct*)reg);
-	}
-
-	postprocadds=0;
-	regs=cfGetProfileString(sec, "postprocadds", "");
-	while (cfGetSpaceListEntry(regname, &regs, 49))
-	{
-		void *reg=_lnkGetSymbol(regname);
-		if (reg)
+		const struct PostProcIntegerRegStruct *postproc = mixAPI->mcpFindPostProcInteger (regname);
+		if (postproc)
 		{
-			((struct mixqpostprocaddregstruct*)reg)->next=postprocadds;
-			postprocadds=(struct mixqpostprocaddregstruct*)reg;
+			fprintf(stderr, "[%s] registering post processing plugin %s\n", driver->name, regname);
+			mixrRegisterPostProc (postproc);
 		}
 	}
+
+	return &devwMix;
 }
 
-static int mixProcKey(unsigned short key)
+static void wmixClose (const struct mcpDriver_t *driver)
 {
-	struct mixqpostprocaddregstruct *mode;
-	for (mode=postprocadds; mode; mode=mode->next)
-	{
-		int r=mode->ProcessKey(key);
-		if (r)
-			return r;
-	}
+	postprocs = 0;
+}
 
-	if (plrProcessKey)
-		return plrProcessKey(key);
+
+static int wmixDetect (const struct mcpDriver_t *driver)
+{
+	return 1;
+}
+
+static int devwMixProcKey (uint16_t key)
+{
+	int i;
+	for (i=0; i < postprocs; i++)
+	{
+		int r = postproc[i]->ProcessKey (key);
+		if (r)
+		{
+			return r;
+		}
+	}
 	return 0;
 }
 
-struct devaddstruct mcpMixAdd = {mixGetOpt, mixrInit, 0, mixProcKey};
-struct sounddevice mcpMixer={SS_WAVETABLE|SS_NEEDPLAYER, 0, "Mixer", wmixDetect, wmixInit, wmixClose, &mcpMixAdd};
-const char *dllinfo="driver mcpMixer";
+static int devwMixFPluginInit (struct PluginInitAPI_t *API)
+{
+	API->mcpRegisterDriver (&mcpMixer);
+	API->mcpRegisterDriver (&mcpMixerQ);
 
-DLLEXTINFO_DRIVER_PREFIX struct linkinfostruct dllextinfo = {.name = "devwmix", .desc = "OpenCP Wavetable Device: Mixer (c) 1994-'22 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", .ver = DLLVERSION, .sortindex = 99};
+	return errOk;
+}
+
+static void devwMixFPluginClose (struct PluginCloseAPI_t *API)
+{
+	API->mcpUnregisterDriver (&mcpMixer);
+	API->mcpUnregisterDriver (&mcpMixerQ);
+}
+
+static const struct mcpDriver_t mcpMixer =
+{
+	"devwMix",
+	"Integer Mixer",
+	wmixDetect,
+	wmixInit,
+	wmixClose
+};
+
+static const struct mcpDriver_t mcpMixerQ =
+{
+	"devwMixQ",
+	"Integer Quality Mixer",
+	wmixDetect,
+	wmixInit,
+	wmixClose
+};
+
+DLLEXTINFO_DRIVER_PREFIX struct linkinfostruct dllextinfo = {.name = "devwmix", .desc = "OpenCP Wavetable Device: Mixer (c) 1994-'23 Niklas Beisert, Tammo Hinrichs, Stian Skjelstad", .ver = DLLVERSION, .sortindex = 99, .PluginInit = devwMixFPluginInit, .PluginClose = devwMixFPluginClose};
