@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Wavetable Device: FPU HighQuality Software Mixer for Pentium/above CPUs
  *
@@ -52,19 +52,21 @@
 #include <unistd.h>
 #include "types.h"
 #include "boot/plinkman.h"
+#include "boot/psetting.h"
 #include "cpiface/cpiface.h"
-#include "dev/imsdev.h"
+#include "dev/deviwave.h"
 #include "dev/mcp.h"
 #include "dev/mix.h"
 #include "dev/player.h"
 #include "dev/plrasm.h"
+#include "dev/postproc.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
-#include "devwmixf.h"
 #include "dwmixfa.h"
 
-/* NB NB   more includes further down in the file */
+static const struct mcpDriver_t mcpFMixer;
 
-extern struct sounddevice mcpFMixer;
+static const struct mixAPI_t *mix;
 
 static int dopause;
 static uint32_t playsamps;
@@ -125,6 +127,7 @@ struct channel
 static struct channel *channels;
 
 static void (*playerproc)(struct cpifaceSessionAPI_t *cpifaceSession);
+static int devwMixFProcKey(uint16_t key);
 
 static uint32_t tickwidth;
 static uint32_t tickplayed;
@@ -330,14 +333,14 @@ static void devwMixFIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 
 	if (dopause)
 	{
-		plrDevAPI->Pause (1);
+		cpifaceSession->plrDevAPI->Pause (1);
 	} else {
 		void *targetbuf;
 		unsigned int targetlength; /* in samples */
 
-		plrDevAPI->Pause (0);
+		cpifaceSession->plrDevAPI->Pause (0);
 
-		plrDevAPI->GetBuffer (&targetbuf, &targetlength);
+		cpifaceSession->plrDevAPI->GetBuffer (&targetbuf, &targetlength);
 
 		while (targetlength)
 		{
@@ -398,21 +401,23 @@ static void devwMixFIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 #ifdef MIXER_DEBUG
 			{
 				int i;
-				fprintf(stderr, "nsamples=%d\n", nsamples);
-				for (i=0;i<nvoices;i++)
-					if (voiceflags[i])
+				fprintf(stderr, "nsamples=%d\n", targetlength);
+				for (i=0;i<channelnum;i++)
+				{
+					if (!(dwmixfa_state.voiceflags[i] & MIXF_QUIET))
 					{
 						fprintf(stderr, "voice #%d\n", i);
 						fprintf(stderr, "  samp: %p (%p)\n", channels[i].samp, channels[i].sbpos);
-						fprintf(stderr, "  looplen: %08x\n", looplen[i]);
-						fprintf(stderr, "  voiceflags: 0x%08x\n", voiceflags[i]);
-						fprintf(stderr, "  freq: 0x%08x:0x%08x\n", freqw[i], freqf[i]);
-						fprintf(stderr, "  vol %lf %lf\n", volleft[i], volright[i]);
+						fprintf(stderr, "  looplen: %08x\n", dwmixfa_state.looplen[i]);
+						fprintf(stderr, "  voiceflags: 0x%08x\n", dwmixfa_state.voiceflags[i]);
+						fprintf(stderr, "  freq: 0x%08x:0x%08x\n", dwmixfa_state.freqw[i], dwmixfa_state.freqf[i]);
+						fprintf(stderr, "  vol %lf %lf\n", dwmixfa_state.volleft[i], dwmixfa_state.volright[i]);
 					}
+				}
 				fprintf(stderr, "\n");
 			}
 #endif
-			mixer();
+			mixer (cpifaceSession);
 
 			tickplayed+=targetlength<<8;
 			if (!((tickwidth-tickplayed)>>8))
@@ -452,9 +457,9 @@ static void devwMixFIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 
 			playsamps += targetlength;
 
-			plrDevAPI->CommitBuffer (targetlength);
+			cpifaceSession->plrDevAPI->CommitBuffer (targetlength);
 
-			plrDevAPI->GetBuffer (&targetbuf, &targetlength);
+			cpifaceSession->plrDevAPI->GetBuffer (&targetbuf, &targetlength);
 
 			if (dopause)
 			{
@@ -463,12 +468,12 @@ static void devwMixFIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 		}
 	}
 
-	IdleCache = plrDevAPI->Idle();
+	IdleCache = cpifaceSession->plrDevAPI->Idle();
 
 	clipbusy--;
 }
 
-static void devwMixFSET(int ch, int opt, int val)
+static void devwMixFSET (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
 {
 	struct channel *chn;
 	if (ch>=channelnum)
@@ -625,7 +630,7 @@ static void devwMixFSET(int ch, int opt, int val)
 			break;
 		case mcpCPitch:
 			chn->orgfrq=8363;
-			chn->orgdiv=mcpGetFreq8363(-val);
+			chn->orgdiv=cpifaceSession->mcpAPI->GetFreq8363(-val);
 			calcstep(chn);
 			break;
 		case mcpCPitchFix:
@@ -689,10 +694,10 @@ static void devwMixFSET(int ch, int opt, int val)
 			calcvols();
 			break;
 		case mcpMasterReverb:
-			masterrvb=(val>=64)?63:(val<-64)?-64:val;
+			masterrvb=(val>=64)?64:(val<0)?0:val;
 			break;
 		case mcpMasterChorus:
-			masterchr=(val>=64)?63:(val<-64)?-64:val;
+			masterchr=(val>=64)?64:(val<0)?0:val;
 			break;
 		case mcpMasterSpeed:
 			relspeed=(val<16)?16:val;
@@ -708,7 +713,7 @@ static void devwMixFSET(int ch, int opt, int val)
 		case mcpMasterAmplify:
 			amplify=val;
 			if (channelnum)
-				mixSetAmplify(amplify);
+				mix->mixSetAmplify (cpifaceSession, amplify);
 			calcvols();
 			break;
 		case mcpMasterPause:
@@ -717,7 +722,7 @@ static void devwMixFSET(int ch, int opt, int val)
 	}
 }
 
-static int devwMixFGET(int ch, int opt)
+static int devwMixFGET (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt)
 {
 /*
 	struct channel *chn;*/
@@ -745,6 +750,18 @@ static int devwMixFGET(int ch, int opt)
 	return 0;
 }
 
+static void devwMixFGetVolRegs (struct cpifaceSessionAPI_t *cpifaceSession, void (*Callback)(struct cpifaceSessionAPI_t *cpifaceSession, const struct ocpvolregstruct *x))
+{
+	int i;
+	for (i=0; i < dwmixfa_state.postprocs; i++)
+	{
+		if (dwmixfa_state.postproc[i]->VolRegs)
+		{
+			Callback (cpifaceSession, dwmixfa_state.postproc[i]->VolRegs);
+		}
+	}
+}
+
 /* Houston, we've got a problem there... the display mixer isn't
  * able to handle floating point values at all. shit.
  * (kebby, irgendwann mal: "ich will das nicht")
@@ -756,7 +773,6 @@ static void GetMixChannel(unsigned int ch, struct mixchannel *chn, uint32_t rate
 {
 	struct channel *c=&channels[ch];
 
-	chn->samp=c->samp;
 	chn->realsamp.fmt=c->samp;
 	chn->length=c->length;
 	chn->loopstart=c->loopstart;
@@ -789,9 +805,9 @@ static void getrealvol(int ch, int *l, int *r)
 	*r=(dwmixfa_state.volr>16319)?255:(dwmixfa_state.volr/64.0);
 }
 
-static int devwMixFLoadSamples(struct sampleinfo *sil, int n)
+static int devwMixFLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct sampleinfo *sil, int n)
 {
-	if (!mcpReduceSamples(sil, n, 0x40000000, mcpRedToMono|mcpRedToFloat|mcpRedNoPingPong))
+	if (!cpifaceSession->mcpAPI->ReduceSamples(sil, n, 0x40000000, mcpRedToMono|mcpRedToFloat|mcpRedNoPingPong))
 		return 0;
 
 #ifdef MIXER_DEBUG
@@ -818,7 +834,7 @@ static int devwMixFOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t
 	if (chan>MIXF_MAXCHAN)
 		chan=MIXF_MAXCHAN;
 
-	if (!plrDevAPI)
+	if (!cpifaceSession->plrDevAPI)
 	{
 		return 0;
 	}
@@ -834,15 +850,15 @@ static int devwMixFOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t
 		goto error_out;
 	}
 
-	currentrate=mcpMixProcRate/chan;
-	dwmixfa_state.samprate=(currentrate>mcpMixMaxRate)?mcpMixMaxRate:currentrate;
+	currentrate = cpifaceSession->mcpAPI->MixProcRate / chan;
+	dwmixfa_state.samprate = ( currentrate > cpifaceSession->mcpAPI->MixMaxRate) ? cpifaceSession->mcpAPI->MixMaxRate : currentrate;
 	format=PLR_STEREO_16BIT_SIGNED;
-	if (!plrDevAPI->Play (&dwmixfa_state.samprate, &format, source_file, cpifaceSession))
+	if (!cpifaceSession->plrDevAPI->Play (&dwmixfa_state.samprate, &format, source_file, cpifaceSession))
 	{
 		goto error_out;
 	}
 
-	if (!mixInit(GetMixChannel, 0, chan, amplify, cpifaceSession))
+	if (!mix->mixInit (cpifaceSession, GetMixChannel, 0, chan, amplify))
 	{
 		goto error_out_plrDevAPI_Play;
 	}
@@ -863,6 +879,7 @@ static int devwMixFOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t
 	cpifaceSession->PhysicalChannelCount = chan;
 	cpifaceSession->mcpGet = devwMixFGET;
 	cpifaceSession->mcpSet = devwMixFSET;
+	cpifaceSession->mcpGetVolRegs = devwMixFGetVolRegs;
 
 	dwmixfa_state.nvoices=channelnum;
 	prepare_mixer();
@@ -872,39 +889,48 @@ static int devwMixFOpenPlayer(int chan, void (*proc)(struct cpifaceSessionAPI_t
 	tickplayed=0;
 	cmdtimerpos=0;
 
+	for (i=0; i < dwmixfa_state.postprocs; i++)
 	{
-		struct mixfpostprocregstruct *mode;
-
-		for (mode=dwmixfa_state.postprocs; mode; mode=mode->next)
-			if (mode->Init) mode->Init(dwmixfa_state.samprate);
+		dwmixfa_state.postproc[i]->Init (dwmixfa_state.samprate);
 	}
+
+	cpifaceSession->mcpActive = 1;
+
 	return 1;
 
-	//mixClose();
+	// mix->mixClose (cpifaceSession);
 error_out_plrDevAPI_Play:
-	plrDevAPI->Stop();
+	cpifaceSession->plrDevAPI->Stop (cpifaceSession);
 error_out:
 	free (dwmixfa_state.tempbuf); dwmixfa_state.tempbuf = 0;
 	free (channels);              channels = 0;
 	return 0;
 }
 
-static void devwMixFClosePlayer()
+static void devwMixFClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	struct mixfpostprocregstruct *mode;
+	int i;
 
-	plrDevAPI->Stop();
+	if (cpifaceSession->plrDevAPI)
+	{
+		cpifaceSession->plrDevAPI->Stop (cpifaceSession);
+	}
 
 	channelnum=0;
 
-	mixClose();
+	mix->mixClose (cpifaceSession);
 
-	for (mode=dwmixfa_state.postprocs; mode; mode=mode->next)
-		if (mode->Close) mode->Close();
+	for (i=0; i < dwmixfa_state.postprocs; i++)
+	{
+		dwmixfa_state.postproc[i]->Close ();
+	}
 
 	free(channels);
 	free(dwmixfa_state.tempbuf);
+	channels = 0;
 	dwmixfa_state.tempbuf = 0;
+
+	cpifaceSession->mcpActive = 0;
 }
 
 static const struct mcpDevAPI_t devwMixF =
@@ -912,13 +938,26 @@ static const struct mcpDevAPI_t devwMixF =
 	devwMixFOpenPlayer,
 	devwMixFLoadSamples,
 	devwMixFIdle,
-	devwMixFClosePlayer
+	devwMixFClosePlayer,
+	devwMixFProcKey
 };
 
-static int Init(const struct deviceinfo *dev)
+static void mixfRegisterPostProc (const struct PostProcFPRegStruct *postproc)
 {
-	volramp=!!(dev->opt&MIXF_VOLRAMP);
-	declick=!!(dev->opt&MIXF_DECLICK);
+	if (dwmixfa_state.postprocs >= MIXF_MAX_POSTPROC)
+	{
+		return;
+	}
+	dwmixfa_state.postproc[dwmixfa_state.postprocs] = postproc;
+	dwmixfa_state.postprocs++;
+}
+
+static const struct mcpDevAPI_t *devwMixFInit (const struct mcpDriver_t *driver, const struct configAPI_t *config, const struct mixAPI_t *mixAPI)
+{
+	char regname[50];
+	const char *regs;
+
+	mix = mixAPI;
 
 	calcinterpoltab();
 
@@ -931,113 +970,71 @@ static int Init(const struct deviceinfo *dev)
 	masterpan=0;
 	mastersrnd=0;
 	channelnum=0;
+	dwmixfa_state.postprocs = 0;
 
-	mcpDevAPI = &devwMixF;
+	volramp = config->GetProfileBool("devwMixF", "volramp", 1, 1);
+	declick = config->GetProfileBool("devwMixF", "declick", 1, 1);
 
-	return 1;
-}
+	fprintf(stderr, "[devwMixF] C version, (volramp=%d, declick=%d)\n", volramp, declick);
 
-static void Close()
-{
-	if (mcpDevAPI == &devwMixF)
+	regs=config->GetProfileString("devwMixF", "postprocs", "");
+	while (config->GetSpaceListEntry(regname, &regs, 49))
 	{
-		mcpDevAPI = 0;
-	}
-}
-
-
-static int Detect(struct deviceinfo *c)
-{
-	c->devtype=&mcpFMixer;
-	c->port=-1;
-	c->port2=-1;
-/*
-	c->irq=-1;
-	c->irq2=-1;
-	c->dma=-1;
-	c->dma2=-1;
-*/
-	if (c->subtype==-1)
-		c->subtype=0;
-	c->chan=MIXF_MAXCHAN;
-	c->mem=0;
-	c->path[0]=0;
-	return 1;
-}
-
-
-
-#include "dev/devigen.h"
-#include "boot/psetting.h"
-#include "dev/deviplay.h"
-#include "boot/plinkman.h"
-
-static struct mixfpostprocaddregstruct *postprocadds;
-
-static uint32_t mixfGetOpt(const char *sec)
-{
-	uint32_t opt=0;
-	if (cfGetProfileBool(sec, "volramp", 1, 1))
-		opt|=MIXF_VOLRAMP;
-	if (cfGetProfileBool(sec, "declick", 1, 1))
-		opt|=MIXF_DECLICK;
-	return opt;
-}
-
-void mixfRegisterPostProc(struct mixfpostprocregstruct *mode)
-{
-	mode->next=dwmixfa_state.postprocs;
-	dwmixfa_state.postprocs=mode;
-}
-
-static void mixfInit(const char *sec)
-{
-	char regname[50];
-	const char *regs;
-
-	fprintf(stderr, "[devwmixf] INIT, ");
-	fprintf(stderr, "using dwmixfa.c C version\n");
-
-	dwmixfa_state.postprocs=0;
-	regs=cfGetProfileString(sec, "postprocs", "");
-
-	while (cfGetSpaceListEntry(regname, &regs, 49))
-	{
-		void *reg=_lnkGetSymbol(regname);
-		if (reg)
-			mixfRegisterPostProc((struct mixfpostprocregstruct*)reg);
-	}
-
-	postprocadds=0;
-	regs=cfGetProfileString(sec, "postprocadds", "");
-	while (cfGetSpaceListEntry(regname, &regs, 49))
-	{
-		void *reg=_lnkGetSymbol(regname);
-		if (reg)
+		const struct PostProcFPRegStruct *postproc = mixAPI->mcpFindPostProcFP (regname);
+		if (postproc)
 		{
-			((struct mixfpostprocaddregstruct*)reg)->next=postprocadds;
-			postprocadds=(struct mixfpostprocaddregstruct*)reg;
+			fprintf(stderr, "[devwMixF] registering post processing plugin %s\n", postproc->name);
+			mixfRegisterPostProc(postproc);
 		}
 	}
+
+	return &devwMixF;
 }
 
-static int mixfProcKey(uint16_t key)
+static void devwMixFClose (const struct mcpDriver_t *driver)
 {
-	struct mixfpostprocaddregstruct *mode;
-	for (mode=postprocadds; mode; mode=mode->next)
-	{
-		int r=mode->ProcessKey(key);
-		if (r)
-			return r;
-	}
+	dwmixfa_state.postprocs = 0;
+}
 
-	if (plrProcessKey)
-		return plrProcessKey(key);
+
+static int devwMixFDetect (const struct mcpDriver_t *driver)
+{
+	return 1;
+}
+
+static int devwMixFProcKey(uint16_t key)
+{
+	int i;
+	for (i=0; i < dwmixfa_state.postprocs; i++)
+	{
+		int r = dwmixfa_state.postproc[i]->ProcessKey (key);
+		if (r)
+		{
+			return r;
+		}
+	}
 	return 0;
 }
 
-struct devaddstruct mcpFMixAdd = {mixfGetOpt, mixfInit, 0, mixfProcKey};
-struct sounddevice mcpFMixer={SS_WAVETABLE|SS_NEEDPLAYER, 0, "FPU Mixer", Detect, Init, Close, &mcpFMixAdd};
-const char *dllinfo="driver mcpFMixer";
+static int devwMixFPluginInit (struct PluginInitAPI_t *API)
+{
+	API->mcpRegisterDriver (&mcpFMixer);
 
-DLLEXTINFO_DRIVER_PREFIX struct linkinfostruct dllextinfo = {.name = "devwmixf", .desc = "OpenCP Wavetable Device: FPU HighQuality Mixer (c) 1999-'22 Tammo Hinrichs, Fabian Giesen, Stian Skjelstad, Jindřich Makovička", .ver = DLLVERSION, .sortindex = 99};
+	return errOk;
+}
+
+static void devwMixFPluginClose (struct PluginCloseAPI_t *API)
+{
+	API->mcpUnregisterDriver (&mcpFMixer);
+}
+
+static const struct mcpDriver_t mcpFMixer =
+{
+	"devwMixF",
+	"FPU Mixer",
+	devwMixFDetect,
+	devwMixFInit,
+	devwMixFClose,
+};
+
+DLLEXTINFO_DRIVER_PREFIX struct linkinfostruct dllextinfo = {.name = "devwmixf", .desc = "OpenCP Wavetable Device: FPU HighQuality Mixer (c) 1999-'23 Hammo Hinrichs, Fabian Giesen, Stian Skjelstad, Jindřich Makovička", .ver = DLLVERSION, .sortindex = 99, .PluginInit = devwMixFPluginInit, .PluginClose = devwMixFPluginClose};
