@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2022 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2022-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Parsing of .CUE files, describing the layout of a CDROM image.
  *
@@ -292,6 +292,11 @@ static void cue_parser_modify_source_LITTLEENDIAN (struct cue_parser_t *cue_pars
 	cue_parser->datasource[cue_parser->datasourceN - 1].swap = 0;
 }
 
+static void cue_parser_modify_source_BINARY (struct cue_parser_t *cue_parser)
+{
+	cue_parser->datasource[cue_parser->datasourceN - 1].swap = 2; // we need to detect
+}
+
 static void cue_parser_modify_source_BIGENDIAN (struct cue_parser_t *cue_parser)
 {
 	cue_parser->datasource[cue_parser->datasourceN - 1].swap = 1;
@@ -370,7 +375,7 @@ static int cue_parse_token (struct cue_parser_t *cue_parser, enum CUE_tokens tok
 		{
 			return -1;
 		}
-		if (index <= cue_parser->track_data[cue_parser->track].index)
+		if (index < cue_parser->track_data[cue_parser->track].index)
 		{
 			return -1;
 		}
@@ -460,7 +465,7 @@ static int cue_parse_token (struct cue_parser_t *cue_parser, enum CUE_tokens tok
 				cue_parser_modify_source_LITTLEENDIAN (cue_parser);
 				break;
 			case CUE_TOKEN_BINARY:
-				cue_parser_modify_source_LITTLEENDIAN (cue_parser);
+				cue_parser_modify_source_BINARY (cue_parser); // should be little, but we need to detect
 				break;
 			case CUE_TOKEN_MOTOROLA:
 				cue_parser_modify_source_BIGENDIAN (cue_parser);
@@ -519,7 +524,7 @@ static int cue_parse_token (struct cue_parser_t *cue_parser, enum CUE_tokens tok
 			default:
 			case CUE_TOKEN_string:
 			case CUE_TOKEN_number:
-				fprintf (stderr, "REM with unknonwn token: %s\n", src);
+				fprintf (stderr, "REM with unknown token: %s\n", src);
 				/* fall-through */
 			case CUE_TOKEN_COMMENT:
 				return 1;
@@ -690,6 +695,13 @@ static int cue_parse_line (struct cue_parser_t *cue_parser, const char *input, c
 	while (l)
 	{
 		const char *current = input;
+
+		if (cue_parser->state == CUE_PARSER_STATE_rem)
+		{
+			cue_parser->state = CUE_PARSER_STATE_ready;
+			return 0;
+		}
+
 		bufferfill = 0;
 		if ((input[0] == ' ') || (input[0] == '\t'))
 		{
@@ -910,7 +922,7 @@ instring:
 	return 0;
 }
 
-void __attribute__ ((visibility ("internal"))) cue_parser_free (struct cue_parser_t *cue_parser)
+OCP_INTERNAL void cue_parser_free (struct cue_parser_t *cue_parser)
 {
 	int i;
 	for (i = 0; i < 100; i++)
@@ -933,7 +945,7 @@ void __attribute__ ((visibility ("internal"))) cue_parser_free (struct cue_parse
 	free (cue_parser);
 }
 
-struct cue_parser_t __attribute__ ((visibility ("internal"))) *cue_parser_from_data (const char *input)
+OCP_INTERNAL struct cue_parser_t *cue_parser_from_data (const char *input)
 {
 	struct cue_parser_t *retval;
 	const char *eof = input + strlen (input);
@@ -973,7 +985,40 @@ struct cue_parser_t __attribute__ ((visibility ("internal"))) *cue_parser_from_d
 	return retval;
 }
 
-struct cdfs_disc_t __attribute__ ((visibility ("internal"))) *cue_parser_to_cdfs_disc (struct ocpfile_t *parentfile, struct cue_parser_t *cue_parser)
+OCP_INTERNAL void detect_endian (const uint8_t *buffer, int *little, int *big)
+{
+	int16_t prev_big_left = 0;
+	int16_t prev_big_right = 0;
+	int16_t prev_little_left = 0;
+	int16_t prev_little_right = 0;
+	uint_fast32_t big_accumulated = 0;
+	uint_fast32_t little_accumulated = 0;
+	int i;
+	for (i=0; i < 588; i++)
+	{
+		int16_t big_left     = buffer[(i<<2)+1] | (buffer[(i<<2)+0] << 8);
+		int16_t big_right    = buffer[(i<<2)+3] | (buffer[(i<<2)+2] << 8);
+		int16_t little_left  = buffer[(i<<2)+0] | (buffer[(i<<2)+1] << 8);
+		int16_t little_right = buffer[(i<<2)+2] | (buffer[(i<<2)+3] << 8);
+		big_accumulated += abs((int32_t)prev_big_left  - big_left);
+		big_accumulated += abs((int32_t)prev_big_right - big_right);
+		little_accumulated += abs((int32_t)prev_little_left  - little_left);
+		little_accumulated += abs((int32_t)prev_little_right - little_right);
+		prev_big_left     = big_left;
+		prev_big_right    = big_right;
+		prev_little_left  = little_left;
+		prev_little_right = little_right;
+	}
+	if (big_accumulated < little_accumulated)
+	{
+		(*big) += 1;
+	} else if (little_accumulated < big_accumulated)
+	{
+		(*little) += 1;
+	}
+}
+
+OCP_INTERNAL struct cdfs_disc_t *cue_parser_to_cdfs_disc (struct ocpfile_t *parentfile, struct cue_parser_t *cue_parser)
 {
 	struct cdfs_disc_t *retval = cdfs_disc_new (parentfile);
 	int i;
@@ -1025,6 +1070,59 @@ struct cdfs_disc_t __attribute__ ((visibility ("internal"))) *cue_parser_to_cdfs
 
 		ms = medium_sector_size (mode);
 		sectorcount = (length + ms - 1) / ms;
+
+		if (cue_parser->datasource[i].swap == 2) // we need to detect endian, broken images out in the wild
+		{
+			int datatracks = 0;
+			int audiotracks_big = 0;
+			int audiotracks_little = 0;
+			int copytrackcounter = trackcounter;
+			uint8_t buffer[2352];
+
+			for (; copytrackcounter <= cue_parser->track; copytrackcounter++)
+			{
+				fprintf (stderr, "track %d(%d) %d\n", copytrackcounter, cue_parser->track_data[copytrackcounter].track_mode);
+				if (cue_parser->track_data[copytrackcounter].datasource > i) break;
+
+				if (cue_parser->track_data[copytrackcounter].track_mode == AUDIO)
+				{
+					int offset = cue_parser->track_data[copytrackcounter].index_data[1].offset;
+					int length = ((copytrackcounter + 1 > cue_parser->track) ||
+					              (cue_parser->track_data[copytrackcounter].datasource != cue_parser->track_data[copytrackcounter+1].datasource)) ?
+					                      sectorcount - cue_parser->track_data[copytrackcounter].index_data[1].offset :
+					                      cue_parser->track_data[copytrackcounter+1].index_data[1].offset - cue_parser->track_data[copytrackcounter].index_data[1].offset;
+					int i;
+					for (i=0; i < 5; i++)
+					{
+						if (i * 75 >= length)
+						{
+							break;
+						}
+						fh->seek_set (fh, (offset + i) * 2352);
+						if (fh->read (fh, buffer, 2352) == 2352)
+						{
+							detect_endian (buffer, &audiotracks_little, &audiotracks_big);
+						}
+					}
+				} else if ((cue_parser->track_data[copytrackcounter].track_mode == MODE1_2352) ||
+				           (cue_parser->track_data[copytrackcounter].track_mode == MODE2_2352))
+				{
+					datatracks++; // safe bet
+					break;
+				}
+			}
+
+			if (datatracks) /* DATA => little endian, no swap needed */
+			{
+				cue_parser->datasource[i].swap = 0;
+			} else if (audiotracks_big > audiotracks_little)
+			{
+				cue_parser->datasource[i].swap = 1;
+			} else {
+				cue_parser->datasource[i].swap = 0;
+			}
+		}
+
 		cdfs_disc_datasource_append (retval,
 		                             discoffset,
 		                             sectorcount,
@@ -1052,13 +1150,21 @@ struct cdfs_disc_t __attribute__ ((visibility ("internal"))) *cue_parser_to_cdfs
 		/* second iteration, add referenced tracks */ 
 		for (; trackcounter <= cue_parser->track; trackcounter++)
 		{
+			uint32_t pregap = 0;
 			if (cue_parser->track_data[trackcounter].datasource > i) break;
 
-#warning REMOVE-ME
-			fprintf (stderr, "Adding track %d, LASTRACK? (trackcounter + 1 >= cue_parser->track) %d, datasource matches next track (cue_parser->track_data[j].datasource != cue_parser->track_data[j+1].datasource=%d\n", trackcounter, (trackcounter + 1 > cue_parser->track), (cue_parser->track_data[trackcounter].datasource != cue_parser->track_data[trackcounter+1].datasource));
-			
+			if (trackcounter == 1)
+			{
+				pregap = cue_parser->track_data[trackcounter].index_data[1].offset;
+			} else {
+				if (cue_parser->track_data[trackcounter].index_data[0].offset && cue_parser->track_data[trackcounter].index_data[1].offset)
+				{
+					pregap = cue_parser->track_data[trackcounter].index_data[1].offset - cue_parser->track_data[trackcounter].index_data[0].offset;
+				}
+			}
+
 			cdfs_disc_track_append (retval,
-			                        0, /* we ignore pregaps, since they require zero insertions in CUE files */
+			                        pregap,
 			                        cue_parser->track_data[trackcounter].index_data[1].offset + discoffset,
 			                        ((trackcounter + 1 > cue_parser->track) ||
 						(cue_parser->track_data[trackcounter].datasource != cue_parser->track_data[trackcounter+1].datasource)) ?
@@ -1072,7 +1178,6 @@ struct cdfs_disc_t __attribute__ ((visibility ("internal"))) *cue_parser_to_cdfs
 			                        0); /* message */
 		}
 
-
 		discoffset += sectorcount;
 	}
 
