@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * database for storing a tree of filenames
  *
@@ -24,11 +24,13 @@
 #include "config.h"
 #include <assert.h>
 #include <fcntl.h>
+#ifdef HAVE_PWD_H
+# include <pwd.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
 #include "types.h"
 #include "boot/console.h"
@@ -36,9 +38,27 @@
 #include "mdb.h"
 #include "boot/psetting.h"
 #include "stuff/compat.h"
+#include "stuff/file.h"
 #include "stuff/poutput.h"
 #include "stuff/utf-8.h"
 
+#ifdef CFHOMEDIR_OVERRIDE
+# define CFHOMEDIR CFHOMEDIR_OVERRIDE
+#else
+# define CFHOMEDIR configAPI.HomePath
+#endif
+
+#ifdef CFDATAHOMEDIR_OVERRIDE
+# define CFDATAHOMEDIR CFDATAHOMEDIR_OVERRIDE
+#else
+# define CFDATAHOMEDIR configAPI->DataHomePath
+#endif
+
+#ifdef MEASURESTR_UTF8_OVERRIDE
+# undef measurestr_utf8
+# define measurestr_utf8(x,y) (y)
+#endif
+
 struct dirdbEntry
 {
 	uint32_t parent;
@@ -72,9 +92,10 @@ struct __attribute__((packed)) dirdbheader
 const char dirdbsigv1[60] = "Cubic Player Directory Data Base\x1B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";
 const char dirdbsigv2[60] = "Cubic Player Directory Data Base\x1B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01";
 
-static struct dirdbEntry *dirdbData=0;
-static uint32_t dirdbNum=0;
-static int dirdbDirty=0;
+static osfile            *dirdbFile;
+static struct dirdbEntry *dirdbData = 0;
+static uint32_t dirdbNum = 0;
+static int dirdbDirty = 0;
 
 static uint32_t dirdbRootChild = DIRDB_NOPARENT;
 static uint32_t dirdbFreeChild = DIRDB_NOPARENT;
@@ -150,43 +171,37 @@ static void dumpdirdb(void)
 }
 #endif
 
-int dirdbInit(void)
+int dirdbInit (const struct configAPI_t *configAPI)
 {
-	char *path;
 	struct dirdbheader header;
-	int f;
 	uint32_t i;
 	int retval;
 	int version;
+	char *dirdbPath;
 
 	dirdbRootChild = DIRDB_NOPARENT;
 	dirdbFreeChild = DIRDB_NOPARENT;
 
-	path = malloc(strlen(cfConfigDir)+11+1);
-	if (!path)
+	dirdbPath = malloc ( strlen(CFDATAHOMEDIR) + 11 + 1);
+	if (!dirdbPath)
 	{
 		fprintf(stderr, "dirdbInit: malloc() failed\n");
 		return 1;
 	}
-	strcpy(path, cfConfigDir);
-	strcat(path, "CPDIRDB.DAT");
+	sprintf (dirdbPath, "%sCPDIRDB.DAT", CFDATAHOMEDIR);
+	fprintf(stderr, "Loading %s .. ", dirdbPath);
 
-	if ((f=open(path, O_RDONLY))<0)
+	dirdbFile = osfile_open_readwrite (dirdbPath, 1, 0);
+	free (dirdbPath);
+	dirdbPath = 0;
+	if (!dirdbFile)
 	{
-		perror("open(cfConfigDir/CPDIRDB.DAT)");
-		free (path);
 		return 1;
 	}
 
-	fprintf(stderr, "Loading %s .. ", path);
-
-	free (path);
-	path = 0;
-
-	if (read(f, &header, sizeof(header))!=sizeof(header))
+	if ( osfile_read (dirdbFile, &header, sizeof(header)) != sizeof(header) )
 	{
 		fprintf(stderr, "No header\n");
-		close(f);
 		return 1;
 	}
 	if (memcmp(header.sig, dirdbsigv1, 60))
@@ -194,7 +209,6 @@ int dirdbInit(void)
 		if (memcmp(header.sig, dirdbsigv2, 60))
 		{
 			fprintf(stderr, "Invalid header\n");
-			close(f);
 			return 1;
 		} else {
 			version = 2;
@@ -215,7 +229,7 @@ int dirdbInit(void)
 	for (i=0; i<dirdbNum; i++)
 	{
 		uint16_t len;
-		if (read(f, &len, sizeof(uint16_t))!=sizeof(uint16_t))
+		if ( osfile_read(dirdbFile, &len, sizeof(uint16_t)) != sizeof(uint16_t) )
 		{
 			goto endoffile;
 		}
@@ -223,11 +237,11 @@ int dirdbInit(void)
 		{
 			len = uint16_little(len);
 
-			if (read(f, &dirdbData[i].parent, sizeof(uint32_t))!=sizeof(uint32_t))
+			if ( osfile_read (dirdbFile, &dirdbData[i].parent, sizeof(uint32_t)) != sizeof(uint32_t) )
 				goto endoffile;
 			dirdbData[i].parent = uint32_little(dirdbData[i].parent);
 
-			if (read(f, &dirdbData[i].mdb_ref, sizeof(uint32_t))!=sizeof(uint32_t))
+			if ( osfile_read (dirdbFile, &dirdbData[i].mdb_ref, sizeof(uint32_t)) != sizeof(uint32_t) )
 				goto endoffile;
 			/* If mdb has been reset, we need to clear all references */
 			dirdbData[i].mdb_ref = mdbCleanSlate ? DIRDB_NO_MDBREF : uint32_little(dirdbData[i].mdb_ref);
@@ -236,14 +250,14 @@ int dirdbInit(void)
 			if (version == 2)
 			{
 				uint32_t discard_adb_ref;
-				if (read(f, &discard_adb_ref, sizeof(uint32_t))!=sizeof(uint32_t))
+				if ( osfile_read (dirdbFile, &discard_adb_ref, sizeof(uint32_t)) != sizeof(uint32_t) )
 					goto endoffile;
 			}
 
 			dirdbData[i].name=malloc(len+1);
 			if (!dirdbData[i].name)
 				goto outofmemory;
-			if (read(f, dirdbData[i].name, len)!=len)
+			if ( osfile_read (dirdbFile, dirdbData[i].name, len) != len)
 			{
 				free(dirdbData[i].name);
 				goto endoffile;
@@ -263,7 +277,6 @@ int dirdbInit(void)
 			/* name is already NULL due to calloc() */
 		}
 	}
-	close(f);
 	for (i=0; i<dirdbNum; i++)
 	{
 		if (dirdbData[i].parent != DIRDB_NOPARENT)
@@ -308,16 +321,20 @@ int dirdbInit(void)
 		}
 	}
 
+#ifdef DIRDB_DEBUG
+	dumpdirdb();
+#endif
+
+	osfile_purge_readaheadcache (dirdbFile);
+
 	fprintf(stderr, "Done\n");
 	return 1;
 endoffile:
 	fprintf(stderr, "EOF\n");
-	close(f);
 	retval=1;
 	goto unload;
 outofmemory:
 	fprintf(stderr, "out of memory\n");
-	close(f);
 	retval=0;
 unload:
 	for (i=0; i<dirdbNum; i++)
@@ -331,19 +348,27 @@ unload:
 		dirdbData[i].next = dirdbFreeChild;
 		dirdbFreeChild = i;
 	}
+	osfile_purge_readaheadcache (dirdbFile);
 	return retval;
 }
 
 void dirdbClose(void)
 {
 	uint32_t i;
+	if (dirdbFile)
+	{
+		osfile_close (dirdbFile);
+		dirdbFile = 0;
+	}
 	if (!dirdbNum)
+	{
 		return;
+	}
 	for (i=0; i<dirdbNum; i++)
 	{
-		free(dirdbData[i].name);
+		free (dirdbData[i].name);
 	}
-	free(dirdbData);
+	free (dirdbData);
 	dirdbData = 0;
 	dirdbNum = 0;
 	dirdbRootChild = DIRDB_NOPARENT;
@@ -356,7 +381,7 @@ uint32_t dirdbFindAndRef(uint32_t parent, char const *name, enum dirdb_use use)
 	struct dirdbEntry *new;
 
 #ifdef DIRDB_DEBUG
-	fprintf(stderr, "dirdbFindAndRef(0x%08x, %s%40s%s, use=%d)\n", parent, name?"\"":"", name?name:"NULL", name?"\"":"", use);
+	fprintf(stderr, "dirdbFindAndRef(0x%08x, %s%40s%s, use=%d) ", parent, name?"\"":"", name?name:"NULL", name?"\"":"", use);
 #endif
 
 	if (!name)
@@ -418,6 +443,7 @@ uint32_t dirdbFindAndRef(uint32_t parent, char const *name, enum dirdb_use use)
 				case dirdb_use_medialib:      dirdbData[i].refcount_medialib++;      break;
 				case dirdb_use_mdb_medialib:  dirdbData[i].refcount_mdb_medialib++;  break;
 			}
+			fprintf (stderr, "=>0x%08x\n", i);
 #endif
 			return i;
 		}
@@ -485,6 +511,7 @@ uint32_t dirdbFindAndRef(uint32_t parent, char const *name, enum dirdb_use use)
 		case dirdb_use_medialib:      dirdbData[i].refcount_medialib++;      break;
 		case dirdb_use_mdb_medialib:  dirdbData[i].refcount_mdb_medialib++;  break;
 	}
+	fprintf (stderr, "=> new 0x%08x (add reference parent)\n", i);
 #endif
 	if (parent!=DIRDB_NOPARENT)
 	{
@@ -500,7 +527,19 @@ uint32_t dirdbFindAndRef(uint32_t parent, char const *name, enum dirdb_use use)
 uint32_t dirdbRef(uint32_t node, enum dirdb_use use)
 {
 #ifdef DIRDB_DEBUG
-	fprintf(stderr, "dirdbRef(0x%08x)\n", node);
+	fprintf(stderr, "dirdbRef(0x%08x) ", node);
+	switch (use)
+	{
+		case dirdb_use_children:      fprintf (stderr, "children");      break;
+		case dirdb_use_dir:           fprintf (stderr, "directories");   break;
+		case dirdb_use_file:          fprintf (stderr, "files");         break;
+		case dirdb_use_filehandle:    fprintf (stderr, "filehandles");   break;
+		case dirdb_use_drive_resolve: fprintf (stderr, "drive_resolve"); break;
+		case dirdb_use_pfilesel:      fprintf (stderr, "pfilesel");      break;
+		case dirdb_use_medialib:      fprintf (stderr, "medialib");      break;
+		case dirdb_use_mdb_medialib:  fprintf (stderr, "mdb_medialib");  break;
+	}
+	fprintf (stderr, "\n");
 #endif
 	if (node == DIRDB_NOPARENT)
 	{
@@ -614,35 +653,17 @@ nodrive:
 		/* test for ~/ */
 		if ((flags & DIRDB_RESOLVE_TILDE_HOME) && (next[0] == '~') && (next[1] == dirsplit))
 		{
-#ifndef __W32__
-			char *home;
-
+#ifndef _WIN32
 			newretval = dirdbFindAndRef (DIRDB_NOPARENT, "file:", TEMP_SPACE);
 			dirdbUnref (retval, TEMP_SPACE);
 			retval = newretval;
 
-			home = getenv("HOME");
-			if (!home)
-			{
-				dirdbUnref (retval, TEMP_SPACE);
-				free (segment);
-				return DIRDB_NOPARENT;
-			}
-			newretval = dirdbResolvePathWithBaseAndRef (retval, home, 0, TEMP_SPACE); /* we are already at the root */
+			newretval = dirdbResolvePathWithBaseAndRef (retval, CFHOMEDIR, 0, TEMP_SPACE); /* we are already at the root */
 			dirdbUnref (retval, TEMP_SPACE);
 			retval = newretval;
 #else
-			#warning This is untested code
-			char *home;
+			newretval = dirdbResolvePathWithBaseAndRef (DIRDB_NOPARENT, CFHOMEDIR, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_WINDOWS_SLASH, TEMP_SPACE);
 
-			home = getenv("HOMEPATH"); /* home is expected to contain DRIVE: */
-			if (home = NULL)
-			{
-				dirdbUnref (retval, TEMP_SPACE);
-				free (segment);
-				return DIRDB_NOPARENT;
-			}
-			newretval = dirdbResolvePathWithBaseAndRef (DIRDB_NOPARENT, home, DIRDB_RESOLVE_DRIVE, TEMP_SPACE);
 			dirdbUnref (retval, TEMP_SPACE);
 			retval = newretval;
 #endif
@@ -653,7 +674,7 @@ nodrive:
 		/* test for ~username */
 		if ((flags & DIRDB_RESOLVE_TILDE_USER) && (next[0] == '~'))
 		{
-#if defined(HAVE_GETPWNAM) && (!defined (__W32__))
+#if defined(HAVE_GETPWNAM) && (!defined (_WIN32))
 			struct passwd *e;
 
 			split = strchr (next, dirsplit);
@@ -953,7 +974,19 @@ void dirdbUnref(uint32_t node, enum dirdb_use use)
 {
 	uint32_t parent, *prev;
 #ifdef DIRDB_DEBUG
-	fprintf(stderr, "dirdbUnref(0x%08x)\n", node);
+	fprintf(stderr, "dirdbUnref(0x%08x) ", node);
+	switch (use)
+	{
+		case dirdb_use_children:      fprintf (stderr, "children");      break;
+		case dirdb_use_dir:           fprintf (stderr, "directories");   break;
+		case dirdb_use_file:          fprintf (stderr, "files");         break;
+		case dirdb_use_filehandle:    fprintf (stderr, "filehandles");   break;
+		case dirdb_use_drive_resolve: fprintf (stderr, "drive_resolve"); break;
+		case dirdb_use_pfilesel:      fprintf (stderr, "pfilesel");      break;
+		case dirdb_use_medialib:      fprintf (stderr, "medialib");      break;
+		case dirdb_use_mdb_medialib:  fprintf (stderr, "mdb_medialib");  break;
+	}
+	fprintf (stderr, "\n");
 #endif
 
 	if (node == DIRDB_NOPARENT)
@@ -1051,6 +1084,9 @@ void dirdbUnref(uint32_t node, enum dirdb_use use)
 
 	if (parent!=DIRDB_NOPARENT)
 	{
+#ifdef DIRDB_DEBUG
+		fprintf (stderr, " unref => remove from parent\n");
+#endif
 		dirdbUnref (parent, dirdb_use_children);
 	}
 }
@@ -1092,7 +1128,7 @@ extern void dirdbGetName_malloc(uint32_t node, char **name)
 	}
 }
 
-static void dirdbGetFullname_malloc_R(uint32_t node, char *name, int nobase)
+static void dirdbGetFullname_malloc_R(uint32_t node, char *name, int nobase, int backslash)
 {
 	if (node == DIRDB_NOPARENT)
 	{
@@ -1105,8 +1141,8 @@ static void dirdbGetFullname_malloc_R(uint32_t node, char *name, int nobase)
 			return;
 		}
 	} else {
-		dirdbGetFullname_malloc_R(dirdbData[node].parent, name, nobase);
-		strcat(name, "/");
+		dirdbGetFullname_malloc_R(dirdbData[node].parent, name, nobase, backslash);
+		strcat(name, backslash ? "\\" : "/");
 	}
 	strcat(name, dirdbData[node].name);
 }
@@ -1149,11 +1185,11 @@ void dirdbGetFullname_malloc(uint32_t node, char **name, int flags)
 	}
 	(*name)[0] = 0;
 
-	dirdbGetFullname_malloc_R (node, *name, flags&DIRDB_FULLNAME_NODRIVE);
+	dirdbGetFullname_malloc_R (node, *name, flags&DIRDB_FULLNAME_NODRIVE, flags&DIRDB_FULLNAME_BACKSLASH);
 
 	if (flags&DIRDB_FULLNAME_ENDSLASH)
 	{
-		strcat(*name, "/");
+		strcat(*name, (flags&DIRDB_FULLNAME_BACKSLASH) ? "\\" : "/");
 	}
 
 	if (strlen(*name) != length)
@@ -1162,20 +1198,19 @@ void dirdbGetFullname_malloc(uint32_t node, char **name, int flags)
 	}
 }
 
-
 void dirdbFlush(void)
 {
-	char *path;
-	int f;
 	uint32_t i;
 	uint32_t max;
 	uint16_t buf16;
 	uint32_t buf32;
 	struct dirdbheader header;
 
-	if (!dirdbDirty)
+	if ((!dirdbDirty) || (!dirdbFile))
 		return;
 
+	osfile_setpos (dirdbFile, 0);
+
 	for (i=0;i<dirdbNum;i++)
 	{
 		if (dirdbData[i].name)
@@ -1192,24 +1227,6 @@ void dirdbFlush(void)
 		}
 	}
 
-	path = malloc(strlen(cfConfigDir)+11+1);
-	if (!path)
-	{
-		fprintf(stderr, "dirdbFlush: malloc() failed\n");
-		return;
-	}
-	strcpy(path, cfConfigDir);
-	strcat(path, "CPDIRDB.DAT");
-
-	if ((f=open(path, O_WRONLY|O_CREAT|O_TRUNC, S_IREAD|S_IWRITE))<0)
-	{
-		perror("open(cfConfigDir/CPDIRDB.DAT)");
-		free (path);
-		return;
-	}
-	free (path);
-	path=0;
-
 	max=0;
 	for (i=0;i<dirdbNum;i++)
 		if (dirdbData[i].name)
@@ -1218,37 +1235,35 @@ void dirdbFlush(void)
 	memcpy(header.sig, dirdbsigv2, sizeof(dirdbsigv2));
 	header.entries=uint32_little(max);
 
-	if (write(f, &header, sizeof(header))!=sizeof(header))
+	if (osfile_write (dirdbFile, &header, sizeof(header)) != sizeof(header) )
 		goto writeerror;
 
 	for (i=0;i<max;i++)
 	{
 		int len=(dirdbData[i].name?strlen(dirdbData[i].name):0);
 		buf16=uint16_little(len);
-		if (write(f, &buf16, sizeof(uint16_t))!=sizeof(uint16_t))
+		if ( osfile_write (dirdbFile, &buf16, sizeof(uint16_t)) != sizeof(uint16_t) )
 			goto writeerror;
 		if (len)
 		{
 			buf32=uint32_little(dirdbData[i].parent);
-			if (write(f, &buf32, sizeof(uint32_t))!=sizeof(uint32_t))
+			if ( osfile_write (dirdbFile, &buf32, sizeof(uint32_t)) != sizeof(uint32_t) )
 				goto writeerror;
 			buf32=uint32_little(dirdbData[i].mdb_ref);
-			if (write(f, &buf32, sizeof(uint32_t))!=sizeof(uint32_t))
+			if ( osfile_write (dirdbFile, &buf32, sizeof(uint32_t)) != sizeof(uint32_t) )
 				goto writeerror;
 #warning remove-me this used to be ADB_REF
 			buf32=0xffffffff; //ADB_REF
-			if (write(f, &buf32, sizeof(uint32_t))!=sizeof(uint32_t))
+			if ( osfile_write (dirdbFile, &buf32, sizeof(uint32_t)) != sizeof(uint32_t) )
 				goto writeerror;
-			if (write(f, dirdbData[i].name, len)!=len)
+			if ( osfile_write (dirdbFile, dirdbData[i].name, len) != len )
 				goto writeerror;
 		}
 	}
-	close(f);
 	dirdbDirty=0;
 	return;
 writeerror:
-	perror("dirdb write()");
-	close(f);
+	{}
 }
 
 uint32_t dirdbGetParentAndRef (uint32_t node, enum dirdb_use use)
@@ -1534,7 +1549,7 @@ void utf8_XdotY_name (const int X, const int Y, char *shortname, const char *sou
 
 	temppath = strdup (source);
 
-	if ((lastdot = rindex(temppath + 1, '.'))) /* we allow files to start with . */
+	if ((lastdot = strrchr(temppath + 1, '.'))) /* we allow files to start with . */
 	{
 		*lastdot = 0; /* modify the source - most easy way around the problem */
 
