@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2022 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2022-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to decode PAK (Westwood and Quake) file archives
  *
@@ -1229,7 +1229,7 @@ static void pak_translate (struct pak_instance_t *self, char *src, char **buffer
 
 	DEBUG_PRINT ("pak_translate %s =>", src);
 
-	temp = rindex (src, '/');
+	temp = strrchr (src, '/');
 	if (temp)
 	{
 		src = temp + 1;
