@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2022 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2022-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Glue logic for fetching and caching data from MusicBrainz
  * online music database (CDROM).
@@ -22,20 +22,21 @@
 #include "config.h"
 #include <assert.h>
 #include <ctype.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <poll.h>
-#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/file.h>
-#include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 #include <cJSON.h>
+#ifdef _WIN32
+# include <windows.h>
+# include <fileapi.h>
+# include <winbase.h>
+# include <shellapi.h>
+#else
+# include <sys/wait.h>
+#endif
 #include "types.h"
 #include "boot/psetting.h"
 #include "dirdb.h"
@@ -46,6 +47,8 @@
 #include "musicbrainz.h"
 #include "pfilesel.h"
 #include "stuff/compat.h"
+#include "stuff/file.h"
+#include "stuff/piperun.h"
 #include "stuff/poutput.h"
 #include "stuff/utf-8.h"
 
@@ -69,7 +72,8 @@ struct musicbrainz_cacheline_t
 
 struct musicbrainz_t
 {
-	int fddb;
+	void *pipehandle;
+	osfile *File;
 	struct timespec lastactive;
 	struct musicbrainz_cacheline_t *cache;
 	int                             cachecount;
@@ -78,11 +82,10 @@ struct musicbrainz_t
 	int                             cachedirtyfrom;
 	struct musicbrainz_queue_t *active;
 	struct musicbrainz_queue_t *head, *tail;
-	int fdout;
-	int fderr;
-	pid_t pid;
-	char out[65536];
-	char err[65536];
+	char out [256*1024];
+	char outo[      16];
+	char err [  2*1024];
+	char erro[      16];
 	int outs;
 	int errs;
 };
@@ -94,7 +97,25 @@ struct musicbrainz_cacheline_sort_t
 	char album[MDB_COMPOSER_LEN];
 }; // used by musicbrainzSetup
 
-struct musicbrainz_t musicbrainz = {-1, {0, 0}, 0, 0, 0, 0, 0, NULL, NULL, NULL, -1, -1, 0};
+struct musicbrainz_t musicbrainz = {
+	0,      /* pipehandle */
+	0,      /* File */
+	{0, 0}, /* lastactive */
+	0,      /* cache */
+	0,      /* cachecount */
+	0,      /* cachesize */
+	0,      /* cachedirty */
+	0,      /* cachedirtyfrom */
+	NULL,   /* active */
+	NULL,   /* head */
+	NULL,   /* tail */
+	{0},    /* out  - stdout buffer */
+	{0},    /* outo - stdout-overflow buffer */
+	{0},    /* err  - stderr buffer */
+	{0},    /* erro - stderr-overflow buffer */
+	0,      /* outs - stdout buffer fill */
+	0       /* errs - stderr buffer fill */
+};
 
 static void musicbrainz_parse_release (cJSON *release, struct musicbrainz_database_h **result);
 static const uint32_t SIZE_PRIVATE      = 0x80000000UL;
@@ -108,80 +129,33 @@ static void musicbrainz_setup_done (void);
 
 static int musicbrainz_spawn (struct musicbrainz_queue_t *t)
 {
-	int fdout[2];
-	int fderr[2];
 	char temp[4096];
 
-	if (pipe (fdout) < 0)
-	{
-		return -1;
-	}
-	if (pipe (fderr) < 0)
-	{
-		close (fdout[0]);
-		close (fdout[1]);
-		return -1;
-	}
-
-	musicbrainz.pid = fork ();
-	if (musicbrainz.pid < 0)
-	{
-		close (fdout[0]);
-		close (fdout[1]);
-		close (fderr[0]);
-		close (fderr[1]);
-		return -1;
-	}
-
-	if (musicbrainz.pid > 0)
-	{
-		close (fdout[1]);
-		close (fderr[1]);
-		fcntl (fdout[0], F_SETFD, FD_CLOEXEC);
-		fcntl (fderr[0], F_SETFD, FD_CLOEXEC);
-		musicbrainz.fdout = fdout[0];
-		musicbrainz.fderr = fderr[0];
-		musicbrainz.outs=0;
-		musicbrainz.errs=0;
-		return 0;
-	}
-
-	close (0); /* stdin */
-	open ("/dev/null", O_RDONLY);
-
-	close (1); /* stdout */
-	if (dup (fdout[1]) != 1)
-	{
-		perror ("dup() failed");
-	}
-
-	close (2); /* stderr */
-	if (dup (fderr[1]) != 2)
-	{
-		perror ("dup() failed");
-	}
-
-	close (fdout[0]);
-	close (fdout[1]);
-	close (fderr[0]);
-	close (fderr[1]);
-
 	snprintf (temp, sizeof (temp), "https://musicbrainz.org/ws/2/discid/%s?inc=recordings+artist-credits&cdstubs=no", t->discid);
 
-	execlp ("curl", "curl",
-		"--max-redirs", "10",
-		"--user-agent", "opencubicplayer/" PACKAGE_VERSION " ( stian.skjelstad@gmail.com )",
-		"--header", "Accept: application/json",
-		"--max-time", "10",
-		"-L", temp,
-		NULL);
-
-	perror ("execve(curl)");
-
-	_exit (1);
+	do
+	{
+		const char *command_line[] =
+		{
+#ifdef _WIN32
+			"curl.exe",
+#else
+			"curl",
+#endif
+			"--max-redirs", "10",
+			"--user-agent", "opencubicplayer/" PACKAGE_VERSION " ( stian.skjelstad@gmail.com )",
+			"--header", "Accept: application/json",
+			"--max-time", "10",
+			"-L", temp,
+			0
+		};
+		musicbrainz.outs = 0;
+		musicbrainz.errs = 0;
+		musicbrainz.pipehandle = ocpPipeProcess_create (command_line);
+		return 0;
+	} while (0);
 }
 
-
 void *musicbrainz_lookup_discid_init (const char *discid, const char *toc, struct musicbrainz_database_h **result)
 {
 	struct musicbrainz_queue_t *t;
@@ -228,6 +202,7 @@ void *musicbrainz_lookup_discid_init (const char *discid, const char *toc, struc
 			}
 
 			{
+
 				cJSON *root = cJSON_ParseWithLength (musicbrainz.cache[i].data, size);
 				{
 					cJSON *releases;
@@ -292,75 +267,10 @@ void *musicbrainz_lookup_discid_init (const char *discid, const char *toc, struc
 		return 0;
 	}
 	musicbrainz.active = t;
+
 	return t;
 }
 
-static void musicbrainz_handlestdout (void)
-{
-	if (musicbrainz.outs == sizeof (musicbrainz.out))
-	{
-		char temp[256];
-		int res;
-		res = read (musicbrainz.fdout, temp, 256);
-		if (res < 0)
-		{
-			return;
-		}
-		if (res == 0)
-		{
-			close (musicbrainz.fdout);
-			musicbrainz.fdout = -1;
-		}
-	} else {
-		int res;
-		res = read (musicbrainz.fdout, musicbrainz.out + musicbrainz.outs, sizeof (musicbrainz.out) - musicbrainz.outs);
-		if (res < 0)
-		{
-			return;
-		}
-		if (res == 0)
-		{
-			close (musicbrainz.fdout);
-			musicbrainz.fdout = -1;
-		} else {
-			musicbrainz.outs += res;
-		}
-	}
-}
-
-static void musicbrainz_handlestderr (void)
-{
-	if (musicbrainz.errs == sizeof (musicbrainz.err))
-	{
-		char temp[256];
-		int res;
-		res = read (musicbrainz.fderr, temp, 256);
-		if (res < 0)
-		{
-			return;
-		}
-		if (res == 0)
-		{
-			close (musicbrainz.fderr);
-			musicbrainz.fderr = -1;
-		}
-	} else {
-		int res;
-		res = read (musicbrainz.fderr, musicbrainz.err + musicbrainz.errs, sizeof (musicbrainz.err) - musicbrainz.errs);
-		if (res < 0)
-		{
-			return;
-		}
-		if (res == 0)
-		{
-			close (musicbrainz.fderr);
-			musicbrainz.fderr = -1;
-		} else {
-			musicbrainz.errs += res;
-		}
-	}
-}
-
 static uint32_t musicbrainz_parse_date (const char *src)
 {
 	uint32_t retval = 0;
@@ -602,6 +512,7 @@ static void musicbrainz_finalize (int retval, struct musicbrainz_database_h **re
 		cJSON *releases;
 		if (!root)
 		{
+			fprintf (stderr, "cJSON_ParseWithLength() failed to parse. Data not valid or truncated\n");
 			return;
 		}
 
@@ -637,7 +548,6 @@ int musicbrainz_lookup_discid_iterate (void *token, struct musicbrainz_database_
 		struct timespec now;
 		struct musicbrainz_queue_t *t;
 
-
 		if (musicbrainz.active)
 		{
 			return 1;
@@ -672,53 +582,44 @@ int musicbrainz_lookup_discid_iterate (void *token, struct musicbrainz_database_
 		return 1;
 	}
 
-	if ((musicbrainz.fdout >= 0) || (musicbrainz.fderr >= 0))
+	if (musicbrainz.pipehandle)
 	{
-		int count = 0;
-		struct pollfd fds[2];
-		if ((musicbrainz.fdout >= 0))
-		{
-			fds[count].fd = musicbrainz.fdout;
-			fds[count].events = POLLIN | POLLHUP;
-			fds[count].revents = 0;
-			count++;
-		}
-		if ((musicbrainz.fderr >= 0))
-		{
-			fds[count].fd = musicbrainz.fderr;
-			fds[count].events = POLLIN | POLLHUP;
-			fds[count].revents = 0;
-			count++;
-		}
-		if (poll (fds, count, 0) >= 0)
-		{
-			if ((musicbrainz.fdout >= 0) && (fds[0].revents))
-			{
-				musicbrainz_handlestdout ();
-			}
-			if ((musicbrainz.fderr >= 0) && (fds[count-1].revents))
-			{
-				musicbrainz_handlestderr ();
-			}
-		}
-	}
+		int a, b, retval;
 
-	if ((musicbrainz.fdout < 0) && (musicbrainz.fderr < 0) && (musicbrainz.pid > 0))
-	{
-		int retval = 0;
-		if (waitpid (musicbrainz.pid, &retval, WNOHANG) != musicbrainz.pid)
+		if (musicbrainz.outs != sizeof (musicbrainz.out))
+		{
+			a = ocpPipeProcess_read_stdout (musicbrainz.pipehandle, musicbrainz.out + musicbrainz.outs, sizeof (musicbrainz.out) - musicbrainz.outs);
+			if (a > 0) musicbrainz.outs += a;
+		} else {
+			a = ocpPipeProcess_read_stdout (musicbrainz.pipehandle, musicbrainz.outo, sizeof (musicbrainz.outo));
+		}
+
+		if (musicbrainz.errs != sizeof (musicbrainz.err))
+		{
+			b = ocpPipeProcess_read_stderr (musicbrainz.pipehandle, musicbrainz.err + musicbrainz.errs, sizeof (musicbrainz.err) - musicbrainz.errs);
+			if (b > 0) musicbrainz.errs += b;
+		} else {
+			b = ocpPipeProcess_read_stderr (musicbrainz.pipehandle, musicbrainz.erro, sizeof (musicbrainz.erro));
+		}
+
+		if ((a >= 0) || (b >= 0))
 		{
 			return 1;
 		}
+
+		retval = ocpPipeProcess_destroy (musicbrainz.pipehandle);
+		musicbrainz.pipehandle = 0;
+
 		clock_gettime (CLOCK_MONOTONIC, &musicbrainz.lastactive);
-		musicbrainz.pid = -1;
+
 		musicbrainz_finalize (retval, result);
+
 		free (musicbrainz.active);
 		musicbrainz.active = 0;
 		return 0;
-	} else {
-		return 1;
 	}
+	fprintf (stderr, "musicbrainz_lookup_discid_iterate() called without a pipe active\n");
+	return 0;
 }
 
 void musicbrainz_lookup_discid_cancel (void *token)
@@ -731,45 +632,45 @@ void musicbrainz_lookup_discid_cancel (void *token)
 	}
 	if (musicbrainz.active == token)
 	{
-		char temp[256];
-		kill (musicbrainz.pid, SIGQUIT);
-		while ((musicbrainz.fdout >= 0) || (musicbrainz.fderr >= 0))
+		assert (musicbrainz.pipehandle);
+
+		ocpPipeProcess_terminate (musicbrainz.pipehandle);
+
+		do
 		{
-			if (musicbrainz.fdout >= 0)
+			int a, b;
+
+			if (musicbrainz.outs != sizeof (musicbrainz.out))
 			{
-				int res;
-				res = read (musicbrainz.fdout, temp, 256);
-				if (res == 0)
-				{
-					close (musicbrainz.fdout);
-					musicbrainz.fdout = -1;
-				}
+				a = ocpPipeProcess_read_stdout (musicbrainz.pipehandle, musicbrainz.out, sizeof (musicbrainz.out) - musicbrainz.outs);
+				if (a > 0) musicbrainz.outs += a;
+			} else {
+				a = ocpPipeProcess_read_stdout (musicbrainz.pipehandle, musicbrainz.outo, sizeof (musicbrainz.outo));
 			}
-			if (musicbrainz.fderr >= 0)
+
+			if (musicbrainz.errs != sizeof (musicbrainz.err))
 			{
-				int res;
-				res = read (musicbrainz.fderr, temp, 256);
-				if (res == 0)
-				{
-					close (musicbrainz.fderr);
-					musicbrainz.fderr = -1;
-				}
+				b = ocpPipeProcess_read_stderr (musicbrainz.pipehandle, musicbrainz.err, sizeof (musicbrainz.err) - musicbrainz.errs);
+				if (b > 0) musicbrainz.errs += b;
+			} else {
+				b = ocpPipeProcess_read_stderr (musicbrainz.pipehandle, musicbrainz.erro, sizeof (musicbrainz.erro));
 			}
-		}
-		while (musicbrainz.pid >= 0)
-		{
-			int retval = 0;
-			if (waitpid (musicbrainz.pid, &retval, WNOHANG) != musicbrainz.pid)
+
+			if ((a >= 0) || (b >= 0))
 			{
 				usleep(10000);
 				continue;
 			}
-			clock_gettime (CLOCK_MONOTONIC, &musicbrainz.lastactive);
-			musicbrainz.pid = -1;
-		}
+		} while (0);
+
+		ocpPipeProcess_destroy (musicbrainz.pipehandle);
+		musicbrainz.pipehandle = 0;
+
+		clock_gettime (CLOCK_MONOTONIC, &musicbrainz.lastactive);
 
 		free (musicbrainz.active);
 		musicbrainz.active = 0;
+
 		return;
 	}
 
@@ -794,34 +695,30 @@ void musicbrainz_lookup_discid_cancel (void *token)
 	}
 }
 
-int musicbrainz_init (void)
+int musicbrainz_init (const struct configAPI_t *configAPI)
 {
 	char *path;
 	char header[64];
 
-	if (musicbrainz.fddb >= 0)
+	if (musicbrainz.File)
 	{
+		fprintf (stderr, "musicbrainz already initialzied\n");
 		return 0;
 	}
 	musicbrainz_setup_init ();
 
-	makepath_malloc (&path, 0, cfConfigDir, "CPMUSBRN.DAT", 0);
+	path = malloc (strlen (configAPI->DataHomePath) + strlen ("CPMUSBRN.DAT") + 1);
+	sprintf (path, "%sCPMUSBRN.DAT", configAPI->DataHomePath);
 	fprintf (stderr, "Loading %s .. ", path);
-	musicbrainz.fddb = open (path, O_RDWR | O_CREAT, S_IREAD|S_IWRITE);
-	if (musicbrainz.fddb < 0)
+	musicbrainz.File = osfile_open_readwrite (path, 1, 0);
+	free (path);
+	path = 0;
+	if (!musicbrainz.File)
 	{
-		fprintf (stderr, "open(%s): %s\n", path, strerror (errno));
-		return 0;
-	}
-	free (path); path = 0;
-
-	if (flock (musicbrainz.fddb, LOCK_EX | LOCK_NB))
-	{
-		fprintf (stderr, "Failed to lock the file (more than one instance?)\n");
 		return 0;
 	}
 
-	if (read (musicbrainz.fddb, &header, sizeof (header)) != sizeof (header))
+	if (osfile_read (musicbrainz.File, &header, sizeof (header)) != sizeof (header))
 	{
 		fprintf (stderr, "Empty database\n");
 		return 1;
@@ -836,7 +733,7 @@ int musicbrainz_init (void)
 	while (1)
 	{
 		char temp[28+8+4];
-		if (read (musicbrainz.fddb, temp, (28 + 8 + 4)) != (28 + 8 + 4))
+		if (osfile_read (musicbrainz.File, temp, (28 + 8 + 4)) != (28 + 8 + 4))
 		{
 			break;
 		}
@@ -865,7 +762,7 @@ int musicbrainz_init (void)
 				fprintf (stderr, "musicbrainz_init: malloc() failed\n");
 				break;
 			}
-			if (read (musicbrainz.fddb, musicbrainz.cache[musicbrainz.cachecount].data, (musicbrainz.cache[musicbrainz.cachecount].size & SIZE_MASK)) != (musicbrainz.cache[musicbrainz.cachecount].size & SIZE_MASK))
+			if (osfile_read (musicbrainz.File, musicbrainz.cache[musicbrainz.cachecount].data, (musicbrainz.cache[musicbrainz.cachecount].size & SIZE_MASK)) != (musicbrainz.cache[musicbrainz.cachecount].size & SIZE_MASK))
 			{
 				free (musicbrainz.cache[musicbrainz.cachecount].data);
 				musicbrainz.cache[musicbrainz.cachecount].data = 0;
@@ -883,7 +780,7 @@ void musicbrainz_done (void)
 {
 	uint64_t pos;
 	int i;
-	if (musicbrainz.fddb < 0)
+	if (!musicbrainz.File)
 	{
 		goto done;
 	}
@@ -896,19 +793,11 @@ void musicbrainz_done (void)
 	}
 	if (!musicbrainz.cachedirtyfrom)
 	{
-		lseek (musicbrainz.fddb, 0, SEEK_SET);
-		while (1)
+		osfile_setpos (musicbrainz.File, 0);
+		if (osfile_write (musicbrainz.File, musicbrainzsigv1, 64) != 64)
 		{
-			if (write (musicbrainz.fddb, musicbrainzsigv1, 64) != 64)
-			{
-				if ((errno != EAGAIN) && (errno != EINTR))
-				{
-					fprintf (stderr, "musicbrainz_done: write #1 failed\n");
-					goto done;
-				}
-			} else {
-				break;
-			}
+			fprintf (stderr, "musicbrainz_done: write #1 failed\n");
+			goto done;
 		}
 	}
 	pos = 64;
@@ -917,7 +806,7 @@ void musicbrainz_done (void)
 		pos += 28 + 8 + 4;
 		pos += musicbrainz.cache[i].size & SIZE_MASK;
 	}
-	lseek (musicbrainz.fddb, pos, SEEK_SET);
+	osfile_setpos (musicbrainz.File, pos);
 	for (; i < musicbrainz.cachecount; i++)
 	{
 		char temp[28+8+4];
@@ -926,47 +815,31 @@ void musicbrainz_done (void)
 		memcpy (temp, musicbrainz.cache[i].discid, 28);
 		memcpy (temp + 28, &lastscan, 8);
 		memcpy (temp + 28 + 8, &size, 4);
-		while (1)
+		if (osfile_write (musicbrainz.File, temp, 28 + 8 + 4) != (28 + 8 + 4))
 		{
-			if (write (musicbrainz.fddb, temp, 28 + 8 + 4) !=(28 + 8 +4))
-			{
-				if ((errno != EAGAIN) && (errno != EINTR))
-				{
-					fprintf (stderr, "musicbrainz_done: write #2 failed\n");
-					goto done;
-				}
-			} else {
-				break;
-			}
+			fprintf (stderr, "musicbrainz_done: write #2 failed\n");
+			goto done;
 		}
-		while (1)
+		if (osfile_write (musicbrainz.File, musicbrainz.cache[i].data, (musicbrainz.cache[i].size & SIZE_MASK)) != (musicbrainz.cache[i].size & SIZE_MASK))
 		{
-			if (write (musicbrainz.fddb, musicbrainz.cache[i].data, (musicbrainz.cache[i].size & SIZE_MASK)) != (musicbrainz.cache[i].size & SIZE_MASK))
-			{
-				if ((errno != EAGAIN) && (errno != EINTR))
-				{
-					fprintf (stderr, "musicbrainz_done: write #3 failed\n");
-					goto done;
-				}
-			} else {
-				break;
-			}
+			fprintf (stderr, "musicbrainz_done: write #3 failed\n");
+			goto done;
 		}
 		pos += 28 + 8 + 4;
 		pos += (musicbrainz.cache[i].size & SIZE_MASK);
 	}
-	if (ftruncate (musicbrainz.fddb, pos))
-	{
-		perror ("ftruncate() failed");
-	}
+	osfile_truncate_at (musicbrainz.File, pos);
 done:
 	for (i=0; i < musicbrainz.cachecount; i++)
 	{
 		free (musicbrainz.cache[i].data);
 	}
 	free (musicbrainz.cache);
-	close (musicbrainz.fddb);
-	musicbrainz.fddb = -1;
+	if (musicbrainz.File)
+	{
+		osfile_close (musicbrainz.File);
+		musicbrainz.File = 0;
+	}
 	musicbrainz.cache = 0;
 	musicbrainz.cachesize = 0;
 	musicbrainz.cachecount = 0;
@@ -1016,7 +889,7 @@ static void musicbrainzSetupDialogDraw (struct musicbrainz_cacheline_sort_t *ent
 {
 	int mlWidth = 55;
 	int mlHeight = 7;
-	int mlTop = (plScrHeight - mlHeight) / 2 ;
+	int mlTop = (plScrHeight - mlHeight) / 2;
 	int mlLeft = (plScrWidth - mlWidth) / 2;
 
 	display_nprintf (mlTop + 0, mlLeft, 0x17, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2); /* +---------+ */
@@ -1151,12 +1024,15 @@ static void musicbrainzSetupDraw (const char *title, int dsel, struct musicbrain
 			}
 		} else {
 			char timebuffer[28];
-			struct tm *thetime;
+			struct tm thetime;
 			time_t inputtime;
 
 			inputtime = musicbrainz.cache[sorted[index].pointsat].lastscan;
-			thetime = localtime (&inputtime);
-			strftime(timebuffer, sizeof (timebuffer), "%d.%m.%Y %H:%M %z(%Z)", thetime);
+			if (!localtime_r (&inputtime, &thetime))
+			{
+				memset (&thetime, 0, sizeof (thetime));
+			}
+			strftime(timebuffer, sizeof (timebuffer), "%d.%m.%Y %H:%M %z(%Z)", &thetime);
 
 			displaychr (mlTop + i, mlLeft + 1, (dsel==index)?0x87:0x07, ' ', 1);
 			displaystr (mlTop + i, mlLeft + 2, (dsel==index)?0x87:0x07, timebuffer, 29); /* includes two exrta space */
@@ -1303,7 +1179,7 @@ static void musicbrainzSetupRun (void **token, const struct DevInterfaceAPI_t *A
 		musicbrainzSetupDraw("MusicBrain Cache DataBase", dsel, sorted);
 		if (dialog)
 		{
-	                musicbrainzSetupDialogDraw (sorted + dsel, epos);
+			musicbrainzSetupDialogDraw (sorted + dsel, epos);
 			while (API->console->KeyboardHit())
 			{
 				int key = API->console->KeyboardGetChar();
@@ -1465,30 +1341,36 @@ static void musicbrainzSetupRun (void **token, const struct DevInterfaceAPI_t *A
 							}
 						} else if (epos == 3)
 						{ /* submit */
-							pid_t pid = fork();
+							char url[1024];
+							int datalen = musicbrainz.cache[sorted[dsel].pointsat].size & SIZE_MASK;
+							char *b = memchr (musicbrainz.cache[sorted[dsel].pointsat].data, ' ', datalen);
+							int tracks = 0;
+							int i;
+#ifndef _WIN32
+							pid_t pid;
+#endif
+							if (b)
+							{
+								char *c = memchr (b + 1, ' ', datalen - (b - musicbrainz.cache[sorted[dsel].pointsat].data) - 1);
+								if (c)
+								{
+									tracks = atoi (b + 1);
+								}
+							}
+							snprintf (url, sizeof (url), "https://musicbrainz.org/cdtoc/attach?id=%s&tracks=%d&toc=%.*s", musicbrainz.cache[sorted[dsel].pointsat].discid, tracks, datalen, musicbrainz.cache[sorted[dsel].pointsat].data);
+							for (i=0; url[i]; i++)
+							{
+								if (url[i] == ' ')
+								{
+									url[i] = '+';
+								}
+							}
+#ifdef _WIN32
+							ShellExecute (0, 0, url, 0, 0, SW_SHOW);
+#else
+							pid = fork();
 							if (pid == 0)
 							{
-								int i;
-								char url[1024];
-								int datalen = musicbrainz.cache[sorted[dsel].pointsat].size & SIZE_MASK;
-								char *b = memchr (musicbrainz.cache[sorted[dsel].pointsat].data, ' ', datalen);
-								int tracks = 0;
-								if (b)
-								{
-									char *c = memchr (b + 1, ' ', datalen - (b - musicbrainz.cache[sorted[dsel].pointsat].data) - 1);
-									if (c)
-									{
-										tracks = atoi (b + 1);
-									}
-								}
-								snprintf (url, sizeof (url), "https://musicbrainz.org/cdtoc/attach?id=%s&tracks=%d&toc=%.*s", musicbrainz.cache[sorted[dsel].pointsat].discid, tracks, datalen, musicbrainz.cache[sorted[dsel].pointsat].data);
-								for (i=0; url[i]; i++)
-								{
-									if (url[i] == ' ')
-									{
-										url[i] = '+';
-									}
-								}
 								for (i=3; i < 1024; i++)
 								{
 									close (i);
@@ -1507,6 +1389,7 @@ static void musicbrainzSetupRun (void **token, const struct DevInterfaceAPI_t *A
 								{
 								}
 							}
+#endif
 						}
 						epos = 0;
 						dialog = 0;
