@@ -1,6 +1,6 @@
 /* OpenCP Module Player
- * Copyright (C) 2001-2005 Russell Marks and Ian Collier.
- * copyright (c) 2005-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * Copyright (C) 2001-'05 Russell Marks and Ian Collier.
+ * copyright (c) 2005-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * This file is based on "aylet" main.c, but is VERY modified.
  *
@@ -52,8 +52,6 @@
 
 #include "config.h"
 #include <assert.h>
-#include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -61,11 +59,11 @@
 #include "types.h"
 
 #include "cpiface/cpiface.h"
-#include "dev/deviplay.h"
 #include "dev/mcp.h"
 #include "dev/player.h"
 #include "dev/ringbuffer.h"
 #include "filesel/filesystem.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
 
 #include "ayplay.h"
@@ -84,11 +82,11 @@ static int debug_output = -1;
 #endif
 
 /* from main.c */
-__attribute__ ((visibility ("internal"))) struct aydata_tag aydata;
-__attribute__ ((visibility ("internal"))) struct time_tag ay_tunetime;
-__attribute__ ((visibility ("internal"))) int ay_track=0;
+OCP_INTERNAL struct aydata_tag aydata;
+OCP_INTERNAL struct time_tag ay_tunetime;
+OCP_INTERNAL int ay_track=0;
 static int new_ay_track=0;
-__attribute__ ((visibility ("internal"))) unsigned long ay_tstates,ay_tsmax;
+OCP_INTERNAL unsigned long ay_tstates, ay_tsmax;
 static int ay_current_reg=0;
 static int done_fade=0;
 static int fadetime=10;     /* fadeout time *after* that in sec, 0=none */
@@ -97,16 +95,13 @@ static int silent_max=4*50; /* max frames of silence before skipping */
 static int ay_looped;
 
 /* the memory is a flat all-RAM 64k */
-__attribute__ ((visibility ("internal"))) unsigned char ay_mem[64*1024];
+OCP_INTERNAL unsigned char ay_mem[64*1024];
 #define AYLET_VER               "0.3"
 #define FRAME_STATES_48         (3500000/50)
 #define FRAME_STATES_128        (3546900/50)
 #define FRAME_STATES_CPC        (4000000/50)
 static int do_cpc=0;
 
-/* options */
-static int ay_inpause;
-
 static unsigned long voll,volr;
 static int bal;
 static int vol;
@@ -152,7 +147,7 @@ static struct aydumpbuffer_delayed_states_t *aydumpbuffer_delayed_states_slot_ge
 	return 0;
 }
 
-void __attribute__ ((visibility ("internal"))) ayGetChans(struct ay_driver_frame_state_t *dst)
+OCP_INTERNAL void ayGetChans (struct ay_driver_frame_state_t *dst)
 {
 	*dst = aydumpbuffer_state_current.aydumpbuffer_states;
 }
@@ -187,14 +182,14 @@ do { \
 } while(0)
 
 /* from main.c */
-unsigned int __attribute__ ((visibility ("internal"))) ay_in(int h,int l)
+OCP_INTERNAL unsigned int ay_in (int h,int l)
 {
 	/* presumably nothing? XXX */
 	return 255;
 }
 
 /* from main.c */
-unsigned int __attribute__ ((visibility ("internal"))) ay_out(int h,int l,int a)
+OCP_INTERNAL unsigned int ay_out (int h,int l,int a)
 {
 	static int cpc_f4=0;
 
@@ -346,7 +341,7 @@ static void mem_init(int track)
 }
 
 /* from main.c */
-static int read_ay_file (struct ocpfilehandle_t *in)
+static int read_ay_file (struct ocpfilehandle_t *in, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	unsigned char *data,*ptr,*ptr2;
 	int tmp,f;
@@ -355,23 +350,26 @@ static int read_ay_file (struct ocpfilehandle_t *in)
 	data_len = in->filesize(in);
 	if (data_len > 1024*1024)
 	{
-		return 0;
+		cpifaceSession->cpiDebug (cpifaceSession, "[AY] File too big\n");
+		return errFormStruc;
 	}
 	data = malloc (data_len);
 	if (!data)
 	{
-		return 0;
+		return errAllocMem;
 	}
 	in->seek_set (in, 0);
 	if (in->read (in, data, data_len) != data_len)
 	{
-		return 0;
+		free (data);
+		return errFileRead;
 	}
 
 	if(memcmp(data,"ZXAYEMUL",8)!=0)
 	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[AY] File signature is not ZXAYEMUL\n");
 		free(data);
-		return 0;
+		return errFormSig;
 	}
 
 	/* for the rest, we don't parse that much; just make copies of the
@@ -381,24 +379,29 @@ static int read_ay_file (struct ocpfilehandle_t *in)
 
 	aydata.tracks=NULL;
 
-#define READWORD(x)     (x)=256*(*ptr++); (x)|=*ptr++
-#define READWORDPTR(x)  READWORD(tmp); \
-		if(tmp>=0x8000) tmp=-0x10000+tmp; \
-		if(ptr-data-2+tmp>=data_len || ptr-data-2+tmp<0) \
-		  { \
-                  free(data); \
-                  if(aydata.tracks) free(aydata.tracks); \
-		  return(0); \
-                  } \
-		(x)=ptr-2+tmp
-#define CHECK_ASCIIZ(x) \
-		if(!memchr((x),0,data+data_len-(x))) \
-		  { \
-                  free(data); \
-                  if(aydata.tracks) free(aydata.tracks); \
-		  return(0); \
-                  }
-
+#define READWORD(x)     do { (x)=256*(*ptr++); (x)|=*ptr++;} while (0)
+#define READWORDPTR(x) do { \
+	READWORD(tmp); \
+	if(tmp>=0x8000) \
+		tmp=-0x10000+tmp; \
+	if(ptr-data-2+tmp>=data_len || ptr-data-2+tmp<0) \
+	{ \
+		cpifaceSession->cpiDebug (cpifaceSession, "[AY] Ran out of data\n"); \
+		free(data); \
+		free(aydata.tracks); \
+		return errFormStruc; \
+	} \
+	(x)=ptr-2+tmp; \
+} while (0)
+#define CHECK_ASCIIZ(x) do { \
+	if(!memchr((x),0,data+data_len-(x))) \
+	{ \
+		cpifaceSession->cpiDebug (cpifaceSession, "[AY] Invalid ASCII string\n"); \
+		free(data); \
+		free(aydata.tracks); \
+		return errFormStruc; \
+	} \
+} while (0)
 	ptr=data+8;
 	aydata.filever=*ptr++;
 	aydata.playerver=*ptr++;
@@ -417,7 +420,7 @@ static int read_ay_file (struct ocpfilehandle_t *in)
 	if((aydata.tracks=malloc(aydata.num_tracks*sizeof(struct ay_track_tag)))==NULL) /* can't happen under glibc */
 	{
 		free(data);
-		return 0;
+		return errAllocMem;
 	}
 
 	for(f=0;f<aydata.num_tracks;f++)
@@ -431,9 +434,10 @@ static int read_ay_file (struct ocpfilehandle_t *in)
 	{
 		if(aydata.tracks[f].data-data+10>data_len-4)
 		{
+			cpifaceSession->cpiDebug (cpifaceSession, "[AY] Track %u/%u not complete\n", f + 1, aydata.num_tracks);
 			free(aydata.tracks);
 			free(data);
-			return 0;
+			return errFormStruc;
 		}
 
 		ptr=aydata.tracks[f].data+10;
@@ -449,7 +453,7 @@ static int read_ay_file (struct ocpfilehandle_t *in)
 
 	aydata.filedata=data;
 	aydata.filelen=data_len;
-	return 1;
+	return errOk;
 }
 
 /* from main.c */
@@ -461,7 +465,7 @@ static void tunetime_reset(void)
 
 /* returns zero if we want to exit the emulation (i.e. exit track) */
 static int silent_for=0;
-int __attribute__ ((visibility ("internal"))) ay_do_interrupt (const struct plrDevAPI_t *plrDevAPI)
+OCP_INTERNAL int  ay_do_interrupt (const struct plrDevAPI_t *plrDevAPI)
 {
 	/* check for fade needed */
 	if(!done_fade && stopafter && ay_tunetime.min*60+ay_tunetime.sec>=stopafter)
@@ -515,7 +519,7 @@ int __attribute__ ((visibility ("internal"))) ay_do_interrupt (const struct plrD
 	return 0;
 }
 
-void __attribute__ ((visibility ("internal"))) ay_driver_frame(int16_t *quad_samples, size_t bytes)
+OCP_INTERNAL void ay_driver_frame (int16_t *quad_samples, size_t bytes)
 {
 	int i;
 
@@ -556,7 +560,7 @@ void __attribute__ ((visibility ("internal"))) ay_driver_frame(int16_t *quad_sam
 	aydumpbuffer_n=bytes / 12;
 }
 
-void __attribute__ ((visibility ("internal"))) aySetMute (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int mute)
+OCP_INTERNAL void aySetMute (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int mute)
 {
 	cpifaceSession->MuteChannel[ch] = mute;
 	switch (ch)
@@ -639,7 +643,7 @@ static void ayIdler (struct cpifaceSessionAPI_t *cpifaceSession)
 	}
 }
 
-void __attribute__ ((visibility ("internal"))) ayIdle (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL void ayIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	if (clipbusy++)
 	{
@@ -647,7 +651,7 @@ void __attribute__ ((visibility ("internal"))) ayIdle (struct cpifaceSessionAPI_
 		return;
 	}
 
-	if (ay_inpause || (ay_looped == 3))
+	if (cpifaceSession->InPause || (ay_looped == 3))
 	{
 		cpifaceSession->plrDevAPI->Pause (1);
 	} else {
@@ -805,6 +809,12 @@ void __attribute__ ((visibility ("internal"))) ayIdle (struct cpifaceSessionAPI_
 						length1-=progress;
 						targetlength--;
 
+						if (length1 < 0)
+						{
+							length2 += length1;
+							length1 = 0;
+						}
+
 						accumulated_target++;
 					} /* while (targetlength && length1) */
 					length1 = length2;
@@ -839,7 +849,7 @@ static void aySetVolume(void)
 		volr=(volr*(64-bal))>>6;
 }
 
-static void aySet (int ch, int opt, int val)
+static void aySet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
 {
 	switch (opt)
 	{
@@ -866,47 +876,54 @@ static void aySet (int ch, int opt, int val)
 	}
 }
 
-static int ayGet (int ch, int opt)
+static int ayGet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt)
 {
 	return 0;
 }
 
-int __attribute__ ((visibility ("internal"))) ayOpenPlayer(struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL int ayOpenPlayer(struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	uint32_t ayRate;
 	enum plrRequestFormat format;
+	int retval;
 
 	aydata.filedata = NULL;
 	aydata.tracks = NULL;
 	aydumpbuffer_n = 0;
 
 	if (!cpifaceSession->plrDevAPI)
-		return 0;
+	{
+		return errPlay;
+	}
 
-	if(!read_ay_file(file)) /* 0 meens error */
-		return 0;
+	if ((retval = read_ay_file (file, cpifaceSession)))
+	{
+		return retval;
+	}
 
-	bzero (aydumpbuffer_delayed_states, sizeof (aydumpbuffer_delayed_states));
+	memset (aydumpbuffer_delayed_states, 0, sizeof (aydumpbuffer_delayed_states));
 
 	ayRate=0;
 	format=PLR_STEREO_16BIT_SIGNED;
 	if (!cpifaceSession->plrDevAPI->Play (&ayRate, &format, file, cpifaceSession))
 	{
+		retval = errPlay;
 		goto errorout_aydata;
 	}
 	sound_freq = ayRate;
 
-	ay_inpause = 0;
 	ay_looped = 0;
 	aybuf = malloc(16384 << 2 /* stereo + 16bit */);
 	if (!aybuf)
 	{
+		retval = errAllocMem;
 		goto errorout_plrDevAPI_Start;
 	}
 
 	aybufpos = cpifaceSession->ringbufferAPI->new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED, 16384);
 	if (!aybufpos)
 	{
+		retval = errAllocMem;
 		goto errorout_aybuf;
 	}
 	aybuffpos=0;
@@ -919,14 +936,15 @@ int __attribute__ ((visibility ("internal"))) ayOpenPlayer(struct ocpfilehandle_
 		ay_track=aydata.num_tracks-1;
 	}
 */
-	bzero (ayMute, sizeof(ayMute));
+	memset (ayMute, 0, sizeof(ayMute));
 
 	if (!sound_init())
 	{
+		retval = errAllocMem;
 		goto errorout_ringbuffer_aybufpos;
 	}
 
-	bzero (&aydumpbuffer_state_current, sizeof (aydumpbuffer_state_current));
+	memset (&aydumpbuffer_state_current, 0, sizeof (aydumpbuffer_state_current));
 	aydumpbuffer_state_current.aydumpbuffer_states.clockrate = 1000000;;
 	aydumpbuffer_state_current.aydumpbuffer_states.channel_a_period = 1;
 	aydumpbuffer_state_current.aydumpbuffer_states.channel_b_period = 1;
@@ -950,9 +968,9 @@ int __attribute__ ((visibility ("internal"))) ayOpenPlayer(struct ocpfilehandle_
 	cpifaceSession->mcpSet=aySet;
 	cpifaceSession->mcpGet=ayGet;
 
-	cpifaceSession->mcpAPI->Normalize (cpifaceSession, mcpNormalizeDefaultPlayP);
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayP);
 
-	return 1;
+	return errOk;
 
 	//sound_end();
 errorout_ringbuffer_aybufpos:
@@ -962,20 +980,23 @@ errorout_aybuf:
 	free(aybuf);
 	aybuf = 0;
 errorout_plrDevAPI_Start:
-	cpifaceSession->plrDevAPI->Stop ();
+	cpifaceSession->plrDevAPI->Stop (cpifaceSession);
 errorout_aydata:
 	free(aydata.tracks);
 	aydata.tracks = 0;
 	free(aydata.filedata);
 	aydata.filedata = 0;
-	return 0;
+	return retval;
 }
 
-void __attribute__ ((visibility ("internal"))) ayClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL void ayClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	sound_end();
 
-	cpifaceSession->plrDevAPI->Stop();
+	if (cpifaceSession->plrDevAPI)
+	{
+		cpifaceSession->plrDevAPI->Stop(cpifaceSession);
+	}
 
 	if (aybufpos)
 	{
@@ -997,22 +1018,17 @@ void __attribute__ ((visibility ("internal"))) ayClosePlayer (struct cpifaceSess
 #endif
 }
 
-int __attribute__ ((visibility ("internal"))) ayIsLooped(void)
+OCP_INTERNAL int ayIsLooped (void)
 {
 	return ay_looped == 3;
 }
 
-void __attribute__ ((visibility ("internal"))) aySetLoop(unsigned char s)
+OCP_INTERNAL void aySetLoop (unsigned char s)
 {
 	donotloop=!s;
 }
 
-void __attribute__ ((visibility ("internal"))) ayPause(unsigned char p)
-{
-	ay_inpause=p;
-}
-
-void __attribute__ ((visibility ("internal"))) ayGetInfo(struct ayinfo *info)
+OCP_INTERNAL void ayGetInfo (struct ayinfo *info)
 {
 	info->track=ay_track+1;
 	info->numtracks=aydata.num_tracks;
@@ -1022,7 +1038,7 @@ void __attribute__ ((visibility ("internal"))) ayGetInfo(struct ayinfo *info)
 }
 
 
-void __attribute__ ((visibility ("internal"))) ayStartSong (struct cpifaceSessionAPI_t *cpifaceSession, int song)
+OCP_INTERNAL void ayStartSong (struct cpifaceSessionAPI_t *cpifaceSession, int song)
 {
 	new_ay_track=song-1;
 	cpifaceSession->ringbufferAPI->reset(aybufpos);
