@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * OGGPlay - Player for Ogg Vorbis files
  *
@@ -36,13 +36,13 @@
 #include <vorbis/vorbisfile.h>
 #include "types.h"
 #include "cpiface/cpiface.h"
-#include "dev/deviplay.h"
 #include "dev/mcp.h"
 #include "dev/player.h"
 #include "dev/plrasm.h"
 #include "dev/ringbuffer.h"
 #include "filesel/filesystem.h"
 #include "oggplay.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
 #include "stuff/poutput.h"
 
@@ -75,8 +75,6 @@ static volatile int active;
 static int ogg_looped;
 static int donotloop;
 
-static int ogg_inpause;
-
 static volatile int clipbusy=0;
 
 static struct ocpfilehandle_t *oggfile;
@@ -143,7 +141,7 @@ static void oggIdler (struct cpifaceSessionAPI_t *cpifaceSession)
 		}
 		if (ov_pcm_tell(&ov)!=oggpos)
 		{
-			fprintf (stderr, "[playogg]: warning, frame position is broken in file (got=%" PRId64 ", expected= %" PRId64 ")\n", ov_pcm_tell(&ov), oggpos);
+			cpifaceSession->cpiDebug (cpifaceSession, "[OGG] warning, frame position is broken in file (got=%" PRId64 ", expected= %" PRId64 ")\n", ov_pcm_tell(&ov), oggpos);
 		}
 
 		cpifaceSession->ringbufferAPI->get_head_samples (oggbufpos, &pos1, &length1, &pos2, &length2);
@@ -170,8 +168,8 @@ static void oggIdler (struct cpifaceSessionAPI_t *cpifaceSession)
 
 			if (result<=0) /* broken data... we can survive */
 			{
-				bzero (oggbuf+(pos1<<1), read<<2); /* always clear 16bit, stereo, signed */
-				fprintf (stderr, "[playogg] ov_read failed: %ld\n", result);
+				memset (oggbuf+(pos1<<1), 0, read<<2); /* always clear 16bit, stereo, signed */
+				cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_read failed: %ld\n", result);
 				result=read;
 			} else {
 				result>>=(1+oggstereo);
@@ -203,7 +201,7 @@ static void oggIdler (struct cpifaceSessionAPI_t *cpifaceSession)
 	}
 }
 
-void __attribute__ ((visibility ("internal"))) oggIdle (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL void oggIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	if (clipbusy++)
 	{
@@ -211,7 +209,7 @@ void __attribute__ ((visibility ("internal"))) oggIdle (struct cpifaceSessionAPI
 		return;
 	}
 
-	if (ogg_inpause || (ogg_looped == 3))
+	if (cpifaceSession->InPause || (ogg_looped == 3))
 	{
 		cpifaceSession->plrDevAPI->Pause (1);
 	} else {
@@ -367,6 +365,12 @@ void __attribute__ ((visibility ("internal"))) oggIdle (struct cpifaceSessionAPI
 						length1-=progress;
 						targetlength--;
 
+						if (length1 < 0)
+						{
+							length2 += length1;
+							length1 = 0;
+						}
+
 						accumulated_target++;
 					} /* while (targetlength && length1) */
 					length1 = length2;
@@ -431,10 +435,10 @@ static long tell_func (void *token)
 }
 
 
-struct ogg_comment_t __attribute__ ((visibility ("internal"))) **ogg_comments;
-int                  __attribute__ ((visibility ("internal")))   ogg_comments_count;
-struct ogg_picture_t __attribute__ ((visibility ("internal")))  *ogg_pictures;
-int                  __attribute__ ((visibility ("internal")))   ogg_pictures_count;
+OCP_INTERNAL struct ogg_comment_t **ogg_comments;
+OCP_INTERNAL int                    ogg_comments_count;
+OCP_INTERNAL struct ogg_picture_t  *ogg_pictures;
+OCP_INTERNAL int                    ogg_pictures_count;
 
 static void add_comment2(const char *title, const char *value)
 {
@@ -746,21 +750,16 @@ static void add_comment (struct cpifaceSessionAPI_t *cpifaceSession, const char
 	free (tmp);
 }
 
-char __attribute__ ((visibility ("internal"))) oggLooped(void)
+OCP_INTERNAL char oggLooped (void)
 {
 	return ogg_looped == 3;
 }
 
-void __attribute__ ((visibility ("internal"))) oggSetLoop(uint8_t s)
+OCP_INTERNAL void oggSetLoop (uint8_t s)
 {
 	donotloop=!s;
 }
 
-void __attribute__ ((visibility ("internal"))) oggPause(uint8_t p)
-{
-	ogg_inpause=p;
-}
-
 static void oggSetSpeed (uint16_t sp)
 {
 	if (sp < 4)
@@ -777,7 +776,7 @@ static void oggSetVolume (void)
 		volr=(volr*(64-bal))>>6;
 }
 
-static void oggSet(int ch, int opt, int val)
+static void oggSet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
 {
 	switch (opt)
 	{
@@ -804,17 +803,17 @@ static void oggSet(int ch, int opt, int val)
 	}
 }
 
-static int oggGet(int ch, int opt)
+static int oggGet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt)
 {
 	return 0;
 }
 
-ogg_int64_t __attribute__ ((visibility ("internal"))) oggGetPos (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL ogg_int64_t oggGetPos (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	return (ogglen + ogglen + oggpos - cpifaceSession->ringbufferAPI->get_tail_available_samples(oggbufpos) - cpifaceSession->plrDevAPI->Idle())%ogglen;
 }
 
-void __attribute__ ((visibility ("internal"))) oggGetInfo (struct cpifaceSessionAPI_t *cpifaceSession, struct ogginfo *info)
+OCP_INTERNAL void oggGetInfo (struct cpifaceSessionAPI_t *cpifaceSession, struct ogginfo *info)
 {
 	static int lastsafe=0;
 	info->pos = oggGetPos (cpifaceSession);
@@ -840,7 +839,7 @@ void __attribute__ ((visibility ("internal"))) oggGetInfo (struct cpifaceSession
 	info->opt50=opt50;
 }
 
-void __attribute__ ((visibility ("internal"))) oggSetPos (struct cpifaceSessionAPI_t *cpifaceSession, ogg_int64_t pos)
+OCP_INTERNAL void oggSetPos (struct cpifaceSessionAPI_t *cpifaceSession, ogg_int64_t pos)
 {
 	pos=(pos+ogglen)%ogglen;
 
@@ -884,15 +883,16 @@ static ov_callbacks callbacks =
 	close_func,
 	tell_func
 };
-int __attribute__ ((visibility ("internal"))) oggOpenPlayer(struct ocpfilehandle_t *oggf, struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL int oggOpenPlayer (struct ocpfilehandle_t *oggf, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	enum plrRequestFormat format;
 	int result;
+	int retval;
 	struct vorbis_info *vi;
 
 	if (!cpifaceSession->plrDevAPI)
 	{
-		return 0;
+		return errPlay;
 	}
 
 	oggfile = oggf;
@@ -902,13 +902,14 @@ int __attribute__ ((visibility ("internal"))) oggOpenPlayer(struct ocpfilehandle
 	{
 		switch (result)
 		{
-			case OV_EREAD:      fprintf (stderr, "ov_open_callbacks(): A read from media returned an error.\n"); break;
-			case OV_ENOTVORBIS: fprintf (stderr, "ov_open_callbacks(): Bitstream does not contain any Vorbis data.\n"); break;
-			case OV_EVERSION:   fprintf (stderr, "ov_open_callbacks(): Vorbis version mismatch.\n"); break;
-			case OV_EBADHEADER: fprintf (stderr, "ov_open_callbacks(): Invalid Vorbis bitstream header.\n"); break;
-			case OV_EFAULT:     fprintf (stderr, "ov_open_callbacks(): Internal logic fault; indicates a bug or heap/stack corruption.\n"); break;
-			default:            fprintf (stderr, "ov_open_callbacks(): Unknown error %d\n", result); break;
+			case OV_EREAD:      cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): A read from media returned an error.\n"); break;
+			case OV_ENOTVORBIS: cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Bitstream does not contain any Vorbis data.\n"); break;
+			case OV_EVERSION:   cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Vorbis version mismatch.\n"); break;
+			case OV_EBADHEADER: cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Invalid Vorbis bitstream header.\n"); break;
+			case OV_EFAULT:     cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Internal logic fault; indicates a bug or heap/stack corruption.\n"); break;
+			default:            cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Unknown error %d\n", result); break;
 		}
+		retval = errFormStruc;
 		goto error_out_file;
 	}
 
@@ -920,7 +921,7 @@ int __attribute__ ((visibility ("internal"))) oggOpenPlayer(struct ocpfilehandle
 	format=PLR_STEREO_16BIT_SIGNED;
 	if (!cpifaceSession->plrDevAPI->Play (&oggRate, &format, oggfile, cpifaceSession))
 	{
-		fprintf(stderr, "playogg: plrOpenPlayer() failed\n");
+		retval = errPlay;
 		goto error_out_file;
 	}
 
@@ -930,17 +931,20 @@ int __attribute__ ((visibility ("internal"))) oggOpenPlayer(struct ocpfilehandle
 	ogglen=ov_pcm_total(&ov, -1);
 	if (!ogglen)
 	{
+		retval = errFormStruc;
 		goto error_out_plrDevAPI_Play;
 	}
 
 	oggbuf=malloc(1024 * 128);
 	if (!oggbuf)
 	{
+		retval = errAllocMem;
 		goto error_out_plrDevAPI_Play;
 	}
 	oggbufpos = cpifaceSession->ringbufferAPI->new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED, 1024*32);
 	if (!oggbufpos)
 	{
+		retval = errAllocMem;
 		goto error_out_oggbuf;
 	}
 	oggbuffpos=0;
@@ -961,20 +965,19 @@ int __attribute__ ((visibility ("internal"))) oggOpenPlayer(struct ocpfilehandle
 		}
 	}
 
-	ogg_inpause=0;
 	ogg_looped=0;
 
 	cpifaceSession->mcpSet = oggSet;
 	cpifaceSession->mcpGet = oggGet;
 
-	cpifaceSession->mcpAPI->Normalize (cpifaceSession, mcpNormalizeDefaultPlayP);
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayP);
 
 	active=1;
 	opt25_50 = 0;
 	opt25[0] = 0;
 	opt50[0] = 0;
 
-	return 1;
+	return errOk;
 
 	//cpifaceSession->ringbufferAPI->free (oggbufpos);
 	//oggbufpos = 0;
@@ -984,7 +987,7 @@ error_out_oggbuf:
 	oggbuf = 0;
 
 error_out_plrDevAPI_Play:
-	cpifaceSession->plrDevAPI->Stop();
+	cpifaceSession->plrDevAPI->Stop (cpifaceSession);
 
 error_out_file:
 	ov_clear(&ov);
@@ -996,14 +999,14 @@ error_out_file:
 		oggfile->unref (oggfile);
 		oggfile = 0;
 	}
-	return 0;
+	return retval;
 }
 
-void __attribute__ ((visibility ("internal"))) oggClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
+OCP_INTERNAL void oggClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	if (active)
 	{
-		cpifaceSession->plrDevAPI->Stop();
+		cpifaceSession->plrDevAPI->Stop (cpifaceSession);
 	}
 	active=0;
 
