@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2011-'22 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2011-'23 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
  *
  * SIDPlay interface routines
  *
@@ -29,8 +29,7 @@
 
 extern "C"
 {
-#include "config.h"
-}
+#include "../config.h"
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,8 +37,6 @@ extern "C"
 #include <time.h>
 #include <math.h>
 #include "types.h"
-extern "C"
-{
 #include "boot/plinkman.h"
 #include "boot/psetting.h"
 #include "cpiface/cpiface.h"
@@ -48,6 +45,7 @@ extern "C"
 #include "filesel/filesystem.h"
 #include "filesel/mdb.h"
 #include "stuff/compat.h"
+#include "stuff/err.h"
 #include "stuff/poutput.h"
 #include "stuff/sets.h"
 #include "sidtype.h"
@@ -57,69 +55,13 @@ extern "C"
 #include "cpiinfo.h"
 #include "sidplay.h"
 
-static time_t starttime;      /* when did the song start, if paused, this is slided if unpaused */
-static time_t pausetime;      /* when did the pause start (fully paused) */
-static time_t pausefadestart; /* when did the pause fade start, used to make the slide */
-static int8_t pausefadedirection; /* 0 = no slide, +1 = sliding from pause to normal, -1 = sliding from normal to pause */
-
-static void togglepausefade (struct cpifaceSessionAPI_t *cpifaceSession)
-{
-	if (pausefadedirection)
-	{ /* we are already in a pause-fade, reset the fade-start point */
-		pausefadestart = clock_ms() - 1000 + (clock_ms() - pausefadestart);
-		pausefadedirection *= -1; /* inverse the direction */
-	} else if (cpifaceSession->InPause)
-	{ /* we are in full pause already */
-		pausefadestart = clock_ms();
-		starttime = starttime + pausefadestart - pausetime; /* we are unpausing, so push starttime the amount we have been paused */
-		sidPause (cpifaceSession->InPause = 0);
-		pausefadedirection = 1;
-	} else { /* we were not in pause, start the pause fade */
-		pausefadestart = clock_ms();
-		pausefadedirection = -1;
-	}
-}
-
-static void dopausefade (struct cpifaceSessionAPI_t *cpifaceSession)
-{
-	int16_t i;
-	if (pausefadedirection > 0)
-	{ /* unpause fade */
-		i = ((int_fast32_t)(clock_ms() - pausefadestart)) * 64 / 1000;
-		if (i < 1)
-		{
-			i = 1;
-		}
-		if (i >= 64)
-		{
-			i = 64;
-			pausefadedirection = 0; /* we reached the end of the slide */
-		}
-	} else { /* pause fade */
-		i = 64 - ((int_fast32_t)(clock_ms() - pausefadestart)) * 64 / 1000;
-		if (i >= 64)
-		{
-			i = 64;
-		}
-		if (i <= 0)
-		{ /* we reached the end of the slide, finish the pause command */
-			pausefadedirection = 0;
-			pausetime = clock_ms();
-			sidPause (cpifaceSession->InPause = 1);
-			return;
-		}
-	}
-	cpifaceSession->mcpAPI->SetMasterPauseFadeParameters (cpifaceSession, i);
-}
-
 static void sidDrawGStrings (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	cpifaceSession->drawHelperAPI->GStringsSongXofY
 	(
 		cpifaceSession,
 		sidGetSong(),
-		sidGetSongs(),
-		cpifaceSession->InPause ? ((pausetime - starttime) / 1000) : ((clock_ms() - starttime) / 1000)
+		sidGetSongs()
 	);
 }
 
@@ -389,21 +331,10 @@ static int sidProcessKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t k
 			cpifaceSession->KeyHelp (KEY_CTRL_HOME, "Next to start of song");
 			return 0;
 		case 'p': case 'P':
-			togglepausefade (cpifaceSession);
+			cpifaceSession->TogglePauseFade (cpifaceSession);
 			break;
 		case KEY_CTRL_P:
-			/* cancel any pause-fade that might be in progress */
-			pausefadedirection = 0;
-			cpifaceSession->mcpAPI->SetMasterPauseFadeParameters (cpifaceSession, 64);
-
-			if (cpifaceSession->InPause)
-			{
-				starttime = starttime + clock_ms() - pausetime;
-			} else {
-				pausetime = clock_ms();
-			}
-			cpifaceSession->InPause = !cpifaceSession->InPause;
-			sidPause (cpifaceSession->InPause);
+			cpifaceSession->TogglePause (cpifaceSession);
 			break;
 		case '<':
 		case KEY_CTRL_LEFT:
@@ -411,7 +342,7 @@ static int sidProcessKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t k
 			if (csg)
 			{
 				sidStartSong(csg);
-				starttime = clock_ms();
+				cpifaceSession->ResetSongTimer (cpifaceSession);
 			}
 			break;
 		case '>':
@@ -420,12 +351,12 @@ static int sidProcessKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t k
 			if (csg<=sidGetSongs())
 			{
 				sidStartSong(csg);
-				starttime = clock_ms();
+				cpifaceSession->ResetSongTimer (cpifaceSession);
 			}
 			break;
 		case KEY_CTRL_HOME:
 			sidStartSong(csg=sidGetSong());
-			starttime = clock_ms();
+			cpifaceSession->ResetSongTimer (cpifaceSession);
 			break;
 		default:
 			return 0;
@@ -435,10 +366,6 @@ static int sidProcessKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t k
 
 static int sidLooped (struct cpifaceSessionAPI_t *cpifaceSession, int LoopMod)
 { /* We do not detect loops at the moment */
-	if (pausefadedirection)
-	{
-		dopausefade (cpifaceSession);
-	}
 	sidIdle (cpifaceSession);
 	return 0;
 }
@@ -446,15 +373,20 @@ static int sidLooped (struct cpifaceSessionAPI_t *cpifaceSession, int LoopMod)
 static int sidOpenFile (struct cpifaceSessionAPI_t *cpifaceSession, struct moduleinfostruct *info, struct ocpfilehandle_t *sidf)
 {
 	const char *filename;
+	int retval;
 
 	if (!sidf)
-		return -1;
+	{
+		return errFormStruc;
+	}
 
 	cpifaceSession->dirdb->GetName_internalstr (sidf->dirdb_ref, &filename);
-	fprintf(stderr, "loading %s...\n", filename);
+	cpifaceSession->cpiDebug (cpifaceSession, "[SID] loading %s...\n", filename);
 
-	if (!sidOpenPlayer(sidf, cpifaceSession))
-		return -1;
+	if ((retval = sidOpenPlayer(sidf, cpifaceSession)))
+	{
+		return retval;
+	}
 
 	cpifaceSession->LogicalChannelCount = sidNumberOfChips() * 3;
 	cpifaceSession->PhysicalChannelCount = sidNumberOfChips() * 4;
@@ -468,13 +400,11 @@ static int sidOpenFile (struct cpifaceSessionAPI_t *cpifaceSession, struct modul
 	cpifaceSession->GetPChanSample = sidGetPChanSample;
 	cpifaceSession->GetLChanSample = sidGetLChanSample;
 
-	starttime = clock_ms();
 	cpifaceSession->InPause = 0;
-	pausefadedirection = 0;
 
 	SidInfoInit (cpifaceSession);
 
-	return 0;
+	return errOk;
 }
 
 static int sidPluginInit (PluginInitAPI_t *API)
@@ -497,15 +427,15 @@ extern "C"
 	DLLEXTINFO_PLAYBACK_PREFIX_CPP struct linkinfostruct dllextinfo =
 	{ /* c++ historically does not support named initializers, and size needs to be writable... */
 		/* .name = */ "playsid",
-		/* .desc = */ "OpenCP SID Player (c) 1993-'22 Michael Schwendt, Tammo Hinrichs, Stian Skjelstad",
+		/* .desc = */ "OpenCP SID Player (c) 1993-'23 Michael Schwendt, Tammo Hinrichs, Stian Skjelstad",
 		/* .ver  = */ DLLVERSION,
 		/* .sortindex = */ 95,
 		/* .PreInit = */ 0,
 		/* .Init = */ 0,
-		/* .LateInit = */ 0,
 		/* .PluginInit = */ sidPluginInit,
-		/* .PluginClose = */ sidPluginClose,
+		/* .LateInit = */ 0,
 		/* .PreClose = */ 0,
+		/* .PluginClose = */ sidPluginClose,
 		/* .Close = */ 0,
 		/* .LateClose = */ 0
 	};
