@@ -1,10 +1,35 @@
+/* OpenCP Module Player
+ * copyright (c) 2022-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ *
+ * Timidity config (setup:) editor
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 #include "config.h"
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
+#ifdef _WIN32
+# include <handleapi.h>
+# include <fileapi.h>
+#else
+# include <sys/stat.h>
+#endif
 #include <unistd.h>
 #include "types.h"
 #include "boot/plinkman.h"
@@ -19,8 +44,8 @@
 #include "stuff/err.h"
 #include "stuff/poutput.h"
 #include "timidityconfig.h"
-#include "timidity-git/timidity/sysdep.h"
 #include "timidity-git/timidity/timidity.h"
+#include "timidity-git/timidity/sysdep.h"
 
 #ifdef PLAYTIMIDITY_DEBUG
 # define PRINT(fmt, args...) fprintf(stderr, "%s %s: " fmt, __FILE__, __func__, ##args)
@@ -28,9 +53,9 @@
 # define PRINT(a, ...) do {} while(0)
 #endif
 
-static int have_user_timidity = 0;
-static char user_timidity_path[BUFSIZ];
-static int  have_default_timidity = 0;
+static char *user_timidity_path = 0;
+static int   have_user_timidity = 0;
+static int   have_default_timidity = 0;
 static char default_timidity_path[BUFSIZ];
 static int    global_timidity_count = 0;
 static char **global_timidity_path = 0;
@@ -69,49 +94,53 @@ static void reset_configfiles (void)
 	}
 	free (global_timidity_path);
 	free (sf2_files_path);
+	free (user_timidity_path);
 	global_timidity_count = 0;
 	global_timidity_path = 0;
 	sf2_files_count = 0;
 	sf2_files_path = 0;
 	have_user_timidity = 0;
-	user_timidity_path[0] = 0;
+	user_timidity_path = 0;
 	have_default_timidity = 0;
 	default_timidity_path[0] = 0;
 }
 
-static void try_user (const char *path)
+static void try_user (const struct DevInterfaceAPI_t *API, const char *filename)
 {
-	struct stat st;
-	if (lstat (path, &st))
+	struct ocpfile_t *f;
+
+	f = ocpdir_readdir_file (API->configAPI->HomeDir, filename, API->dirdb);
+	if (!f)
 	{
-		PRINT ("Unable to lstat() user config via $HOME %s\n", path);
-		return;
-	}
-	if ((st.st_mode & S_IFMT) == S_IFLNK)
-	{
-		if (stat (path, &st))
-		{
-			PRINT ("Unable to stat() user config via $HOME %s, broken symlink\n", path);
-			return;
-		}
-	}
-	if ((st.st_mode & S_IFMT) != S_IFREG)
-	{
-		PRINT ("Unable to use user config via $HOME %s, not a regular file\n", path);
+		PRINT ("Unable to use user config via cfHOME %s\n", filename);
 		return;
 	}
 	if (have_user_timidity)
 	{
-		PRINT ("Unable to use user config via $HOME %s, already have one\n", path);
-		return;
+		PRINT ("Already have a user config\n");
+	} else {
+		PRINT ("Found user config via $HOME %s\n", filename);
+		API->dirdb->GetFullname_malloc (f->dirdb_ref, &user_timidity_path, 0);
 	}
-	PRINT ("Found user config via $HOME %s\n", path);
-	have_user_timidity = 1;
-	snprintf (user_timidity_path, sizeof (user_timidity_path), "%s", path);
+	f->unref (f);
 }
 
 static void try_global (const char *path)
 {
+#ifdef _WIN32
+	DWORD st;
+
+	st = GetFileAttributes (path);
+
+	if (st == INVALID_FILE_ATTRIBUTES)
+	{
+		return;
+	}
+	if (st & FILE_ATTRIBUTE_DIRECTORY)
+	{
+		return;
+	}
+#else
 	struct stat st;
 	if (lstat (path, &st))
 	{
@@ -131,6 +160,7 @@ static void try_global (const char *path)
 		PRINT ("Unable to use global config %s, not a regular file\n", path);
 		return;
 	}
+#endif
 	PRINT ("Found global config %s%s", path, have_user_timidity?", with possible user overrides":" (no user overrides found earlier)\n");
 	if (!have_default_timidity)
 	{
@@ -141,6 +171,42 @@ static void try_global (const char *path)
 
 static void scan_config_directory (const char *dpath)
 {
+#ifdef _WIN32
+	HANDLE FindHandle;
+	WIN32_FIND_DATAA FindData;
+	char path[BUFSIZ];
+
+	FindHandle = FindFirstFile (dpath, &FindData);
+	if (FindHandle == INVALID_HANDLE_VALUE)
+	{
+		PRINT ("Unable to scan directory %s for global configuration files\n", dpath);
+		return;
+	}
+
+	do
+	{
+		snprintf (path, sizeof (path), "%s%s%s", dpath, PATH_STRING, FindData.cFileName);
+
+		if (FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+		{
+			PRINT ("Ignoring %s, since it is a directory\n", path);
+			continue;
+		}
+		if (strlen (FindData.cFileName) < 4)
+		{
+			PRINT ("Ignoring %s, since it does not end with .cfg (too short)\n", path);
+			continue;
+		}
+		if (strcasecmp (FindData.cFileName + strlen (FindData.cFileName) - 4, ".cfg"))
+		{
+			PRINT ("Ignoring %s, since it does not end with .cfg\n", path);
+			continue;
+		}
+		PRINT ("Found global config %s\n", path);
+		append_global (path);
+	} while (FindNextFile (FindHandle, &FindData));
+	FindClose (FindHandle);
+#else
 	DIR *d = opendir (dpath);
 	struct dirent *de;
 	if (!d)
@@ -186,10 +252,47 @@ static void scan_config_directory (const char *dpath)
 		append_global (path);
 	}
 	closedir (d);
+#endif
 }
 
 static void scan_sf2_directory (const char *dpath)
 {
+#ifdef _WIN32
+	HANDLE FindHandle;
+	WIN32_FIND_DATAA FindData;
+	char path[BUFSIZ];
+
+	FindHandle = FindFirstFile (dpath, &FindData);
+	if (FindHandle == INVALID_HANDLE_VALUE)
+	{
+		PRINT ("Unable to scan directory %s for global sf2 fonts\n", dpath);
+		return;
+	}
+
+	do
+	{
+		snprintf (path, sizeof (path), "%s%s%s", dpath, PATH_STRING, FindData.cFileName);
+
+		if (FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+		{
+			PRINT ("Ignoring %s, since it is a directory\n", path);
+			continue;
+		}
+		if (strlen (FindData.cFileName) < 4)
+		{
+			PRINT ("Ignoring %s, since it does not end with .sf2 (too short)\n", path);
+			continue;
+		}
+		if (strcasecmp (FindData.cFileName + strlen (FindData.cFileName) - 4, ".sf2"))
+		{
+			PRINT ("Ignoring %s, since it does not end with .sf2\n", path);
+			continue;
+		}
+		PRINT ("Found sf2 %s\n", path);
+		append_sf2 (path);
+	} while (FindNextFile (FindHandle, &FindData));
+	FindClose (FindHandle);
+#else
 	DIR *d = opendir (dpath);
 	struct dirent *de;
 	if (!d)
@@ -235,6 +338,7 @@ static void scan_sf2_directory (const char *dpath)
 		append_sf2 (path);
 	}
 	closedir (d);
+#endif
 }
 
 static int mystrcmp(const void *a, const void *b)
@@ -242,30 +346,31 @@ static int mystrcmp(const void *a, const void *b)
 	return strcmp (*(char **)a, *(char **)b);
 }
 
-static void refresh_configfiles (void)
+static void refresh_configfiles (const struct DevInterfaceAPI_t *API)
 {
-	char *home;
-	char path[BUFSIZ];
-
 	reset_configfiles ();
 
-	home = getenv("HOME");
-#ifdef __W32__
-/* HOME or home */
-	if(home == NULL)
-		home = getenv("HOMEPATH");
-	if(home == NULL)
-		home = getenv("home");
+#ifdef _WIN32
+	try_user (API, "timidity.cfg");
+	try_user (API, "_timidity.cfg");
 #endif
-	if (home)
-	{
-#ifdef __W32__
-		snprintf (path, sizeof (path), "%s" PATH_STRING "timidity.cfg", home); try_user (path);
-		snprintf (path, sizeof (path), "%s" PATH_STRING "_timidity.cfg", home); try_user (path);
-#endif
-		snprintf (path, sizeof (path), "%s" PATH_STRING ".timidity.cfg", home); try_user (path);
-	}
+	try_user (API, "timidity.cfg");
 
+#ifdef __W32
+	try_global ("C:\\WINDOWS\\timidity.cfg");
+	try_global ("C:\\timidity\\timidity.cfg");
+	try_global ("C:\\TiMidity++\\timidity.cfg");
+	scan_config_directory ("C:\\timidity\\*");
+	scan_config_directory ("C:\\TiMidity++\\*");
+	scan_config_directory ("C:\\timidity\\soundfonts\\*");
+	scan_config_directory ("C:\\TiMidity++\\soundfonts\\*");
+	scan_config_directory ("C:\\timidity\\musix\\*");
+	scan_config_directory ("C:\\TiMidity++\\musix\\*");
+	scan_sf2_directory ("C:\\timidity\\soundfonts\\*");
+	scan_sf2_directory ("C:\\TiMidity++\\soundfonts\\*");
+	scan_sf2_directory ("C:\\timidity\\musix\\*");
+	scan_sf2_directory ("C:\\TiMidity++\\musix\\*");
+#else
 	if (strcmp(CONFIG_FILE, "/etc/timidity/timidity.cfg") &&
 	    strcmp(CONFIG_FILE, "/etc/timidity.cfg") &&
 	    strcmp(CONFIG_FILE, "/usr/local/share/timidity/timidity.cfg") &&
@@ -281,6 +386,7 @@ static void refresh_configfiles (void)
 	scan_config_directory ("/usr/share/timidity");
 	scan_sf2_directory ("/usr/local/share/sounds/sf2");
 	scan_sf2_directory ("/usr/share/sounds/sf2");
+#endif
 
 	if (global_timidity_count >= 2)
 	{
@@ -304,6 +410,7 @@ static void timidityConfigFileSelectDraw (int dsel, const struct DevInterfaceAPI
 	int dot;
 	int contentsel;
 	int contentheight = 6 + global_timidity_count + sf2_files_count + (!global_timidity_count) + (!sf2_files_count);
+	int s;
 #define LINES_NOT_AVAILABLE 5
 
 	half = (mlHeight - LINES_NOT_AVAILABLE) / 2;
@@ -335,7 +442,7 @@ static void timidityConfigFileSelectDraw (int dsel, const struct DevInterfaceAPI
 		dot = skip * (mlHeight - LINES_NOT_AVAILABLE) / (contentheight - (mlHeight - LINES_NOT_AVAILABLE));
 	}
 
-	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2);
+	s = (mlWidth - 2 - 38) / 2; API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4 Select TiMidity++ configuration file %*C\xc4\xbf", s, mlWidth - 2 - 38 - s);
 	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o Please select a new configuration file using the arrow keys and press%*C %0.9o\xb3", mlWidth - 72);
 	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%0.15o <ENTER>%0.7o when done, or %0.15o<ESC>%0.7o to cancel.%*C %.9o\xb3", mlWidth - 41);
 	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xc3%*C\xc4\xb4", mlWidth - 2);
@@ -484,31 +591,31 @@ static int DefaultDelay;
 static int DefaultChorus;
 static void timidityConfigDraw (int EditPos, const struct DevInterfaceAPI_t *API)
 {
-	int large;
+	int large, s;
 	int mlWidth, mlHeight, mlTop, mlLeft;
 	const char *configfile = API->configAPI->GetProfileString ("timidity", "configfile", "");
 	const char *reverbs[] = {"disable", "original", "global-original", "freeverb", "global-freeverb"};
 	const char *effect_lr_modes[] = {"disable", "left", "right", "both"};
 	const char *disable_enable[] = {"disable", "enable"};
 
-	if (API->console->TextHeight >= 43)
+	if (API->console->TextHeight >= 45)
 	{
 		large = 2;
-		mlHeight = 41;
-	} else if (API->console->TextHeight >= 35)
+		mlHeight = 43;
+	} else if (API->console->TextHeight >= 37)
 	{
 		large = 1;
-		mlHeight = 33;
+		mlHeight = 35;
 	} else {
 		large = 0;
-		mlHeight = 23;
+		mlHeight = 24;
 	}
 
 	mlWidth = 70;
 	mlTop = (API->console->TextHeight - mlHeight) / 2;
 	mlLeft = (API->console->TextWidth - mlWidth) / 2;
 
-	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2);
+	s = (mlWidth - 2 - 26) / 2; API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4 TiMidity++ configuration %*C\xc4\xbf", s, mlWidth - 2 - 26 - s);
 
 	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
@@ -516,6 +623,10 @@ static void timidityConfigDraw (int EditPos, const struct DevInterfaceAPI_t *API
 
 	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
+	API->console->DisplayPrintf        (mlTop++, mlLeft, 0x09, mlWidth, "\xc3%*C\xc4\xb4", mlWidth - 2);
+
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+
 	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 1. TiMdity+ Configfile/Soundfont: %.15o<ENTER>%.7o to change%*C %.9o\xb3", mlWidth - 53 - 1);
 
 	if (!configfile[0])
@@ -621,7 +732,7 @@ static void timidityConfigRun (void **token, const struct DevInterfaceAPI_t *API
 {
 	int esel = 0;
 
-	refresh_configfiles ();
+	refresh_configfiles (API);
 
 	DefaultReverbMode     = API->configAPI->GetProfileInt ("timidity", "reverbmode",       2, 10);
 	DefaultReverbLevel    = API->configAPI->GetProfileInt ("timidity", "reverblevel",     40, 10);
@@ -925,7 +1036,7 @@ superexit:
 static struct ocpfile_t *timidityconfig; // needs to overlay an dialog above filebrowser, and after that the file is "finished"   Special case of DEVv
 static void timidityConfigRun (void **token, const struct DevInterfaceAPI_t *API);
 
-int __attribute__ ((visibility ("internal"))) timidity_config_init (struct PluginInitAPI_t *API)
+OCP_INTERNAL int timidity_config_init (struct PluginInitAPI_t *API)
 {
 	timidityconfig = API->dev_file_create (
 		API->dmSetup->basedir,
@@ -944,7 +1055,7 @@ int __attribute__ ((visibility ("internal"))) timidity_config_init (struct Plugi
 	return errOk;
 }
 
-void __attribute__ ((visibility ("internal"))) timidity_config_done (struct PluginCloseAPI_t *API)
+OCP_INTERNAL void timidity_config_done (struct PluginCloseAPI_t *API)
 {
 	if (timidityconfig)
 	{
