@@ -1,5 +1,5 @@
 /* opencp module Player
- * copyright (c) 2018-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2018-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * SDL2 graphic driver
  *
@@ -22,18 +22,20 @@
 #include "config.h"
 #include <assert.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <time.h>
 #include <SDL.h>
 #include "types.h"
 #include "boot/console.h"
 #include "boot/psetting.h"
 #include "cpiface/cpiface.h"
-#include "framelock.h"
-#include "poutput.h"
-#include "poutput-fontengine.h"
-#include "poutput-keyboard.h"
-#include "poutput-sdl2.h"
-#include "poutput-swtext.h"
+#include "stuff/framelock.h"
+#include "stuff/imsrtns.h"
+#include "stuff/poutput.h"
+#include "stuff/poutput-fontengine.h"
+#include "stuff/poutput-keyboard.h"
+#include "stuff/poutput-sdl2.h"
+#include "stuff/poutput-swtext.h"
 
 typedef enum
 {
@@ -92,8 +94,10 @@ static SDL_Texture *current_texture = NULL;
 static int last_text_height;
 static int last_text_width;
 
-static void (*set_state)(int fullscreen, int width, int height) = 0;
-static int do_fullscreen = 0;
+static int cachemode = -1;
+
+static void (*set_state)(const int fullscreen, int width, int height, const int window_resized) = 0;
+static int current_fullsceen = 0;
 static int plScrRowBytes = 0;
 static int ekbhit_sdl2dummy(void);
 static int sdl2_SetGraphMode (int high);
@@ -129,6 +133,9 @@ static uint8_t *virtual_framebuffer = 0;
 
 static void sdl2_close_window(void)
 {
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "[SDL2-video] sdl_close_window()");
+#endif
 	if (current_texture)
 	{
 		SDL_DestroyTexture (current_texture);
@@ -284,10 +291,10 @@ static void sdl2_dump_renderer (void)
 #endif
 }
 
-static void set_state_textmode(int fullscreen, int width, int height)
+static void set_state_textmode (const int fullscreen, int width, int height, const int window_resized)
 {
 #ifdef SDL2_DEBUG
-	fprintf (stderr, " set_state_textmode(fullscreen=%d, width=%d, height=%d\n", fullscreen, width, height);
+	fprintf (stderr, " set_state_textmode(fullscreen=%d, width=%d, height=%d, window_resized=%d)\n", fullscreen, width, height, window_resized);
 #endif
 	/* texture WILL for sure resize, so just get rid of it! */
 	if (current_texture)
@@ -302,10 +309,10 @@ static void set_state_textmode(int fullscreen, int width, int height)
 		Console.VidMem = virtual_framebuffer = 0;
 	}
 
-	if (fullscreen != do_fullscreen)
+	if (fullscreen != current_fullsceen)
 	{
 #ifdef SDL2_DEBUG
-		fprintf (stderr, " fullscreen =! do_fullscreen\n");
+		fprintf (stderr, " fullscreen =! current_fullsceen\n");
 #endif
 		if (fullscreen)
 		{
@@ -323,61 +330,64 @@ static void set_state_textmode(int fullscreen, int width, int height)
 		}
 	}
 
-	if ((fullscreen != do_fullscreen) || (!current_window))
+	if (!width)
 	{
 #ifdef SDL2_DEBUG
-		fprintf (stderr, " destroy and recreate window\n");
+		fprintf (stderr, "[SDL2-video] set_state_textmode() width==0 ???\n");
 #endif
-		sdl2_close_window();
+		width = 640;
+	}
 
-		do_fullscreen = fullscreen;
+	if (!height)
+	{
+#ifdef SDL2_DEBUG
+		fprintf (stderr, "[SDL2-video] set_state_textmode() height==0 ???\n");
+#endif
+		height = 480;
+	}
+
+	/* if the call is due to window resized, do not do any changes to the window */
+	if (!window_resized)
+	{
+		current_fullsceen = fullscreen;
 
 		if (fullscreen)
 		{
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   0, 0,
-		                                           SDL_WINDOW_FULLSCREEN_DESKTOP);
-			if (current_window)
+			if (!current_window)
 			{
-				SDL_GetWindowSize (current_window, &width, &height);
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   0, 0,
+				                                   SDL_WINDOW_FULLSCREEN_DESKTOP);
+			} else {
+				SDL_SetWindowFullscreen (current_window, SDL_WINDOW_FULLSCREEN_DESKTOP);
 			}
 		} else {
-			if (!width)
+			if (!current_window)
 			{
-#ifdef SDL2_DEBUG
-				fprintf (stderr, "[SDL2-video] set_state_textmode() width==0 ???\n");
-#endif
-				width = 640;
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   width, height,
+				                                   SDL_WINDOW_RESIZABLE);
+			} else {
+				SDL_SetWindowFullscreen (current_window, 0);
+				SDL_SetWindowResizable (current_window, SDL_TRUE);
+				SDL_SetWindowSize (current_window, width, height);
 			}
-			if (!height)
-			{
-#ifdef SDL2_DEBUG
-				fprintf (stderr, "[SDL2-video] set_state_textmode() height==0 ???\n");
-#endif
-				height = 480;
-			}
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   SDL_WINDOWPOS_UNDEFINED,
-		                                           width, height,
-		                                           SDL_WINDOW_RESIZABLE);
 		}
-
-		if (!current_window)
-		{
-#ifdef SDL2_DEBUG
-			fprintf (stderr, "[SDL2-video]: SDL_CreateWindow: %s (fullscreen=%d %dx%d)\n", SDL_GetError(), fullscreen, width, height);
-#endif
-			SDL_ClearError();
-		}
-	} else {
-#ifdef SDL2_DEBUG
-		fprintf (stderr, "Leave window\n");
-#endif
 	}
 
+	if (!current_window)
+	{
+		fprintf (stderr, "[SDL2-video]: SDL_CreateWindow: %s (fullscreen=%d %dx%d)\n", SDL_GetError(), fullscreen, width, height);
+		SDL_ClearError();
+		exit(1);
+	}
+
+	SDL_GetWindowSize (current_window, &width, &height);
+
 	while ( ((width/FontSizeInfo[Console.CurrentFont].w) < 80) || ((height/FontSizeInfo[Console.CurrentFont].h) < 25))
 	{
 #ifdef SDL2_DEBUG
@@ -467,33 +477,39 @@ static void sdl2_SetTextMode(unsigned char x)
 
 	if ((x == Console.CurrentMode) && (current_window))
 	{
-		bzero (virtual_framebuffer, Console.GraphBytesPerLine * Console.GraphLines);
+		memset (virtual_framebuffer, 0, Console.GraphBytesPerLine * Console.GraphLines);
 		return;
 	}
 
-	sdl2_SetGraphMode (-1); /* closes the window, so gdb helper below will have a clear screen */
-
 	if (x==255)
 	{
 #ifdef SDL2_DEBUG
-		fprintf (stderr, "gdb helper, plScrMode=255\n", x);
+		fprintf (stderr, "gdb helper, plScrMode=255\n");
 #endif
+		sdl2_SetGraphMode (-1); /* closes the window, so gdb helper below will have a clear screen */
+
 		Console.CurrentMode = 255;
 		return; /* gdb helper */
 	}
 
+	if (cachemode >= 0)
+	{
+		sdl2_SetGraphMode (-1); /* closes the window */
+	}
+
 	/* if invalid mode, set it to custom */
 	if (x>7)
 	{
 #ifdef SDL2_DEBUG
-		fprintf (stderr, "custom request\n", x);
+		fprintf (stderr, "custom request\n");
 #endif
 		x=8;
 
-		set_state_textmode(
-			do_fullscreen,
+		set_state_textmode (
+			current_fullsceen,
 			last_text_width,
-			last_text_height);
+			last_text_height,
+		        0);
 	} else {
 #ifdef SDL2_DEBUG
 		fprintf (stderr, "plCurrentFont=%d (mode_tui_data[%d].font))\n", mode_tui_data[x].font, x);
@@ -504,24 +520,27 @@ static void sdl2_SetTextMode(unsigned char x)
 
 		Console.CurrentFont = mode_tui_data[x].font;
 
-		set_state_textmode(
-			do_fullscreen,
+		set_state_textmode (
+			current_fullsceen,
 			mode_gui_data[mode_tui_data[x].gui_mode].width,
-			mode_gui_data[mode_tui_data[x].gui_mode].height);
+			mode_gui_data[mode_tui_data[x].gui_mode].height,
+		        0);
 	}
 
 	Console.LastTextMode = Console.CurrentMode = x;
 #ifdef SDL2_DEBUG
-	fprintf (stderr, "plScrType = plScrMode = %d\n", Console.LastTextMode );
+	fprintf (stderr, "[SDL2-video] plScrType = plScrMode = %d\n", Console.LastTextMode );
 #endif
 }
 
-static int cachemode = -1;
-
-static void set_state_graphmode(int fullscreen, int width, int height)
+static void set_state_graphmode (const int fullscreen, int width, int height, const int window_resized)
 {
 	mode_gui_t mode;
 
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "[SDL2-video] set_state_graphmode(fullscreen=%d, width=%d, height=%d, window_resized=%d)", fullscreen, width, height, window_resized);
+#endif
+
 	/* texture WILL for sure resize, so just get rid of it! */
 	if (current_texture)
 	{
@@ -550,28 +569,46 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 	width = mode_gui_data[mode].width;
 	height = mode_gui_data[mode].height;
 
-	if ((fullscreen != do_fullscreen) || (!current_window))
+	/* if the call is due to window resized, do not do any changes to the window */
+	if (!window_resized)
 	{
-		sdl2_close_window();
-
-		do_fullscreen = fullscreen;
+		current_fullsceen = fullscreen;
 
 		if (fullscreen)
 		{
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   SDL_WINDOWPOS_UNDEFINED,
-		                                           0, 0,
-		                                           SDL_WINDOW_FULLSCREEN_DESKTOP);
+			if (!current_window)
+			{
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   SDL_WINDOWPOS_UNDEFINED,
+			                                           0, 0,
+			                                           SDL_WINDOW_FULLSCREEN_DESKTOP);
+			} else {
+				SDL_SetWindowFullscreen (current_window, SDL_WINDOW_FULLSCREEN_DESKTOP);
+			}
 		} else {
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-		                                           SDL_WINDOWPOS_UNDEFINED,
-		                                           SDL_WINDOWPOS_UNDEFINED,
-		                                           width, height,
-		                                           0);
+			if (!current_window)
+			{
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+			                                           SDL_WINDOWPOS_UNDEFINED,
+			                                           SDL_WINDOWPOS_UNDEFINED,
+			                                           width, height,
+			                                           0);
+			} else {
+				SDL_SetWindowFullscreen (current_window, 0);
+				SDL_SetWindowResizable (current_window, SDL_FALSE);
+				SDL_SetWindowSize (current_window, width, height);
+			}
 		}
 	}
 
+	if (!current_window)
+	{
+		fprintf (stderr, "[SDL2-video]: SDL_CreateWindow: %s (fullscreen=%d %dx%d)\n", SDL_GetError(), fullscreen, width, height);
+		SDL_ClearError();
+		exit(1);
+	}
+
 	if (!current_renderer)
 	{
 		current_renderer = SDL_CreateRenderer (current_window, -1, 0);
@@ -620,6 +657,10 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 
 static int sdl2_SetGraphMode (int high)
 {
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "[SDL2-video] sdl2_SetGraphMode(high=%d)", high);
+#endif
+
 	if (high>=0)
 		set_state = set_state_graphmode;
 
@@ -632,21 +673,20 @@ static int sdl2_SetGraphMode (int high)
 		free (virtual_framebuffer);
 		Console.VidMem = virtual_framebuffer = 0;
 	}
-	sdl2_close_window();
 
 	if (high<0)
 	{
 		return 0;
 	}
 
-	set_state_graphmode(do_fullscreen, 0, 0);
+	set_state_graphmode (current_fullsceen, 0, 0, 0);
 
 	Console.VidMem = virtual_framebuffer = malloc (Console.GraphBytesPerLine * Console.GraphLines);
 
 quick:
 	if (virtual_framebuffer)
 	{
-		bzero (virtual_framebuffer, Console.GraphBytesPerLine * Console.GraphLines);
+		memset (virtual_framebuffer, 0, Console.GraphBytesPerLine * Console.GraphLines);
 	}
 
 	sdl2_gFlushPal ();
@@ -672,7 +712,7 @@ static void sdl2_DisplaySetupTextMode(void)
 	while (1)
 	{
 		uint16_t c;
-		bzero (virtual_framebuffer, Console.GraphBytesPerLine * Console.GraphLines);
+		memset (virtual_framebuffer, 0, Console.GraphBytesPerLine * Console.GraphLines);
 		make_title("sdl2-driver setup", 0);
 		swtext_displaystr_cp437(1, 0, 0x07, "1:  font-size:", 14);
 		swtext_displaystr_cp437(1, 15, Console.CurrentFont == _8x8 ? 0x0f : 0x07, "8x8", 3);
@@ -694,8 +734,8 @@ static void sdl2_DisplaySetupTextMode(void)
 			case '1':
 				/* we can assume that we are in text-mode if we are here */
 				sdl2_CurrentFontWanted = Console.CurrentFont = (Console.CurrentFont == _8x8)?_8x16:_8x8;
-				set_state_textmode(do_fullscreen, Console.GraphBytesPerLine, Console.GraphLines);
-				cfSetProfileInt("x11", "font", Console.CurrentFont, 10);
+				set_state_textmode (current_fullsceen, Console.GraphBytesPerLine, Console.GraphLines, 0);
+				cfSetProfileInt(cfScreenSec, "fontsize", Console.CurrentFont, 10);
 				break;
 			case KEY_EXIT:
 			case KEY_ESC: return;
@@ -707,7 +747,7 @@ static const char *sdl2_GetDisplayTextModeName(void)
 {
 	static char mode[48];
 	snprintf(mode, sizeof(mode), "res(%dx%d), font(%s)%s", Console.TextWidth, Console.TextHeight,
-		Console.CurrentFont == _8x8 ? "8x8" : "8x16", do_fullscreen?" fullscreen":"");
+		Console.CurrentFont == _8x8 ? "8x8" : "8x16", current_fullsceen?" fullscreen":"");
 	return mode;
 }
 
@@ -1312,18 +1352,18 @@ static int ekbhit_sdl2dummy(void)
 						if (current_window && (SDL_GetWindowID(current_window) == event.window.windowID))
 						{
 							Console.CurrentFont = sdl2_CurrentFontWanted;
-							if (!do_fullscreen && (Console.CurrentMode == Console.LastTextMode ))
+							if (!current_fullsceen && (Console.CurrentMode == Console.LastTextMode ))
 							{
 								last_text_height = event.window.data2;
 								last_text_width = event.window.data1;
 							}
-							set_state(do_fullscreen, event.window.data1, event.window.data2);
-							if (!do_fullscreen)
+							set_state (current_fullsceen, event.window.data1, event.window.data2, 1);
+							if (!current_fullsceen)
 							{
 								if (Console.LastTextMode == Console.CurrentMode) /* if we are in text-mode, make it a custom one */
 								{
 #ifdef SDL2_DEBUG
-									fprintf (stderr, "CUSTOM MODE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+									fprintf (stderr, "[SDL2-video] CUSTOM MODE plScrType = plScrMode = 8\n");
 #endif
 									Console.LastTextMode = Console.CurrentMode = 8;
 								}
@@ -1354,7 +1394,7 @@ static int ekbhit_sdl2dummy(void)
 						break;
 
 					case 3:
-						set_state (!do_fullscreen, Console.GraphBytesPerLine, Console.GraphLines);
+						set_state (!current_fullsceen, Console.GraphBytesPerLine, Console.GraphLines, 0);
 						break;
 
 					case 4:
@@ -1392,7 +1432,9 @@ static int ekbhit_sdl2dummy(void)
 #endif
 				if (skipone)
 				{
-					fprintf (stderr, "skipone hit..\n");
+#ifdef SDL2_DEBIG
+					fprintf (stderr, "              skipone hit..\n");
+#endif
 					skipone=0;
 				} else {
 					int i;
@@ -1498,7 +1540,7 @@ static int ekbhit_sdl2dummy(void)
 					/* TODO, handle ALT-ENTER */
 					if (event.key.keysym.sym==SDLK_RETURN)
 					{
-						set_state (!do_fullscreen, Console.GraphBytesPerLine, Console.GraphLines);
+						set_state (!current_fullsceen, Console.GraphBytesPerLine, Console.GraphLines, 0);
 					} else {
 						for (index=0;translate_alt[index].OCP!=0xffff;index++)
 							if (translate_alt[index].SDL==event.key.keysym.sym)
@@ -1608,13 +1650,13 @@ int sdl2_init(void)
 	SDL_EventState (SDL_TEXTEDITING, SDL_ENABLE);
 
 	/* Fill some default font and window sizes */
-	sdl2_CurrentFontWanted = Console.CurrentFont = cfGetProfileInt("x11", "font", _8x16, 10);
+	sdl2_CurrentFontWanted = Console.CurrentFont = cfGetProfileInt(cfScreenSec, "fontsize", _8x16, 10);
 	if (Console.CurrentFont > _FONT_MAX)
 	{
 		Console.CurrentFont = _8x16;
 	}
-	last_text_width  = Console.GraphBytesPerLine = 640;
-	last_text_height = Console.GraphLines        = 480;
+	last_text_width  = Console.GraphBytesPerLine = saturate(cfGetProfileInt(cfScreenSec, "winwidth",  1280, 10), 640, 16384);
+	last_text_height = Console.GraphLines        = saturate(cfGetProfileInt(cfScreenSec, "winheight", 1024, 10), 400, 16384);
 	Console.LastTextMode = Console.CurrentMode = 8;
 
 	need_quit = 1;
