@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2005-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2005-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * X11 graphic driver
  *
@@ -23,6 +23,8 @@
  *    -first release
  */
 
+#define ENABLE_SHM 1 /* makes it possible to disable XShmImage API, to test the classic non-shm path */
+
 #define _CONSOLE_DRIVER
 #include "config.h"
 #include <ctype.h>
@@ -35,9 +37,13 @@
 #include <X11/xpm.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/xf86vmode.h>
+#ifdef ENABLE_SHM
 #include <X11/extensions/XShm.h>
+#endif
 #include <sys/ipc.h>
+#ifdef ENABLE_SHM
 #include <sys/shm.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -51,14 +57,15 @@
 #include "boot/console.h"
 #include "boot/psetting.h"
 #include "cpiface/cpiface.h"
-#include "framelock.h"
-#include "poutput.h"
-#include "poutput-fontengine.h"
-#include "poutput-keyboard.h"
-#include "poutput-swtext.h"
-#include "poutput-x11.h"
-#include "poll.h"
-#include "x11-common.h"
+#include "stuff/framelock.h"
+#include "stuff/imsrtns.h"
+#include "stuff/poutput.h"
+#include "stuff/poutput-fontengine.h"
+#include "stuff/poutput-keyboard.h"
+#include "stuff/poutput-swtext.h"
+#include "stuff/poutput-x11.h"
+#include "stuff/poll.h"
+#include "stuff/x11-common.h"
 #include "desktop/opencubicplayer-48x48.xpm"
 
 /* TEXT-MODE DRIVER */
@@ -71,8 +78,8 @@ static void create_image(void);
 static void destroy_image(void);
 
 static XSizeHints Textmode_SizeHints;
-static unsigned long Textmode_Window_Width = 0;
-static unsigned long Textmode_Window_Height = 0;
+static unsigned long Textmode_Window_Width;
+static unsigned long Textmode_Window_Height;
 
 static void *x11_TextOverlayAddBGRA (unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned int pitch, uint8_t *data_bgra);
 static void x11_TextOverlayRemove(void *handle);
@@ -80,6 +87,22 @@ static void x11_TextOverlayRemove(void *handle);
 static XIM im;
 static XIC ic;
 
+struct modes_t
+{
+	int charx, chary, windowx, windowy, bigfont/*, modeline*/;
+};
+static const struct modes_t modes[8]=
+{
+	{  80,  25,  640,  400, 1/*, MODE_640x400*/},
+	{  80,  30,  640,  480, 1/*, MODE_640x480*/},
+	{  80,  50,  640,  400, 0/*, MODE_640x400*/},
+	{  80,  60,  640,  480, 0/*, MODE_640x480*/},
+	{ 128,  48, 1024,  768, 1/*, MODE_1024x768*/},
+	{ 160,  64, 1280, 1024, 1/*, MODE_1280x1024*/},
+	{ 128,  96, 1024,  768, 0/*, MODE_1024x768*/},
+	{ 160, 128, 1280, 1024, 0/*, MODE_1280x1024*/}
+};
+
 static const struct consoleDriver_t x11ConsoleDriver;
 
 /* "stolen" from Mplayer START */
@@ -134,8 +157,10 @@ static Atom WM_DELETE_WINDOW;
 static int we_have_fullscreen;
 static int do_fullscreen=0;
 
+#ifdef ENABLE_SHM
 static XShmSegmentInfo shminfo[1];
 static int shm_completiontype = -1;
+#endif
 
 static Window window=0;
 static XImage *image=0;
@@ -158,8 +183,8 @@ static void xvidmode_init(void)
 
 	Console.CurrentFont = _8x8;
 
-	bzero (modelines, sizeof(modelines));
-	bzero (&default_modeline, sizeof(default_modeline));
+	memset (modelines, 0, sizeof(modelines));
+	memset (&default_modeline, 0, sizeof(default_modeline));
 
 	XGetWindowAttributes(mDisplay, DefaultRootWindow(mDisplay), &xwa);
 	fprintf(stderr, "[x11] rootwindow: width:%d height:%d\n", xwa.width, xwa.height);
@@ -375,7 +400,7 @@ static void motif_decoration(Window window, int action)
 	{
 		MotifWmHints vo_MotifWmHints;
 
-		bzero (&vo_MotifWmHints, sizeof(MotifWmHints));
+		memset (&vo_MotifWmHints, 0, sizeof(MotifWmHints));
 		vo_MotifWmHints.flags=MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
 		if (action)
 		{
@@ -440,10 +465,16 @@ static void WindowResized_Textmode(unsigned int width, unsigned int height)
 		virtual_framebuffer = 0;
 		Console.VidMem = (uint8_t *)image->data;
 	}
-	bzero (Console.VidMem, Console.GraphBytesPerLine * Console.GraphLines);
+	memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
 
 	if (!do_fullscreen)
 	{
+		if ((Console.CurrentMode != 8) &&
+		    ((modes[Console.CurrentMode].windowy != height) ||
+		     (modes[Console.CurrentMode].windowx != width)))
+		{
+			Console.LastTextMode = Console.CurrentMode = 8;
+		}
 		Textmode_Window_Height = height;
 		Textmode_Window_Width = width;
 	}
@@ -548,7 +579,7 @@ static void set_state_graphmode(int FullScreen)
 	} else {
 		virtual_framebuffer = 0;
 		Console.VidMem = (uint8_t *)image->data;
-		bzero (Console.VidMem, Console.GraphBytesPerLine * Console.GraphLines);
+		memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
 	}
 }
 
@@ -564,6 +595,8 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 	do_fullscreen = FullScreen;
 
 	Textmode_SizeHints.flags=PMinSize|PMaxSize|USSize|PSize;
+	Textmode_SizeHints.width=Textmode_Window_Width;
+	Textmode_SizeHints.height=Textmode_Window_Height;
 	Textmode_SizeHints.max_width=default_modeline.hdisplay;
 	Textmode_SizeHints.max_height=default_modeline.vdisplay;
 
@@ -671,7 +704,7 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 		virtual_framebuffer = 0;
 		Console.VidMem = (uint8_t *)image->data;
 	}
-	bzero (Console.VidMem, Console.GraphBytesPerLine * Console.GraphLines);
+	memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
 }
 
 static void x11_common_event_loop(void)
@@ -704,8 +737,10 @@ static void x11_common_event_loop(void)
 
 		XNextEvent(mDisplay, &event);
 
+#ifdef ENABLE_SHM
 		if (event.type==shm_completiontype)
 			continue;
+#endif
 
 		switch (event.type)
 		{
@@ -1097,6 +1132,7 @@ static void create_window(void)
 
 static void create_image(void)
 {
+#ifdef ENABLE_SHM
 	if (mLocalDisplay && XShmQueryExtension(mDisplay) )
 	{
 		if (image)
@@ -1138,17 +1174,74 @@ static void create_image(void)
 		shmctl(shminfo[0].shmid, IPC_RMID, 0);
 	} else {
 		shm_completiontype = -1;
+#else
+	{
+#endif
+#if 0 /* XGetImage can fail, typically if window has been just resized into a smaller size */
+		fprintf (stderr, "XGetImage (mDisplay=%p, window=%p, x=0, y=0, width=%d, lines=%d, AllPlanes, ZPixmap)\n", mDisplay, window, Console.GraphBytesPerLine, Console.GraphLines);
 		if (!(image = XGetImage (mDisplay, window, 0, 0, Console.GraphBytesPerLine, Console.GraphLines, AllPlanes, ZPixmap)))
 		{
 			fprintf(stderr, "[x11] Failed to create XImage\n");
 			exit(-1);
 		}
+#else
+
+		int depth = XDefaultDepth(mDisplay, XDefaultScreen(mDisplay));
+		int bpp;
+		void *data;
+		int pad;
+		int bytes_per_line;
+		pad = BitmapPad(mDisplay);
+		switch (depth)
+		{
+			case 1:
+			case 2:
+			case 4:
+			case 8:
+				bpp = 1;
+				break;
+			case 15:
+			case 16:
+				bpp = 2;
+				break;
+			default:
+			case 24:
+			case 32:
+				bpp = 4;
+				break;
+		}
+		bytes_per_line = ((Console.GraphLines + pad - 1) & ~(pad-1)) * bpp;
+		data = calloc (Console.GraphBytesPerLine, bytes_per_line);
+		if (!data)
+		{
+			fprintf (stderr, "calloc() before XCreateImage() failed\n");
+			exit(-1);
+		}
+		if (!(image = XCreateImage (
+			mDisplay,
+			XDefaultVisual(mDisplay, mScreen),
+			depth,
+			ZPixmap,
+			0,
+			data,
+			Console.GraphBytesPerLine,
+			Console.GraphLines,
+			pad,
+			0)))
+		{
+			fprintf(stderr, "[x11] XCreateImage failed\n");
+			exit(-1);
+		}
+
+
+#endif
 	}
 	x11_depth = image->bits_per_pixel;
 }
 
 static void destroy_image(void)
 {
+#ifdef ENABLE_SHM
 	if (shm_completiontype>=0)
 	{
 		if (image)
@@ -1157,7 +1250,9 @@ static void destroy_image(void)
 			XDestroyImage(image);
 			shmdt(shminfo[0].shmaddr);
 		}
-	} else {
+	} else
+#endif
+	{
 		if (image)
 			XDestroyImage(image);
 	}
@@ -1280,10 +1375,10 @@ static int x11_SetGraphMode (int high)
 		Console.VidMem = (uint8_t *)image->data;
 	}
 quick:
-	bzero (image->data, image->bytes_per_line * Console.GraphLines);
+	memset (image->data, 0, image->bytes_per_line * Console.GraphLines);
 	if (virtual_framebuffer)
 	{
-		bzero (virtual_framebuffer, Console.GraphBytesPerLine * Console.GraphLines);
+		memset (virtual_framebuffer, 0, Console.GraphBytesPerLine * Console.GraphLines);
 	}
 
 	x11_gFlushPal ();
@@ -1297,22 +1392,6 @@ static void x11_vga13(void)
 
 static void x11_SetTextMode (unsigned char x)
 {
-	struct modes_t
-	{
-		int charx, chary, windowx, windowy, bigfont/*, modeline*/;
-	};
-	const struct modes_t modes[8]=
-	{
-		{  80,  25,  640,  400, 1/*, MODE_640x400*/},
-		{  80,  30,  640,  480, 1/*, MODE_640x480*/},
-		{  80,  50,  640,  400, 0/*, MODE_640x400*/},
-		{  80,  60,  640,  480, 0/*, MODE_640x480*/},
-		{ 128,  48, 1024,  768, 1/*, MODE_1024x768*/},
-		{ 160,  64, 1280, 1024, 1/*, MODE_1280x1024*/},
-		{ 128,  96, 1024,  768, 0/*, MODE_1024x768*/},
-		{ 160, 128, 1280, 1024, 0/*, MODE_1280x1024*/}
-	};
-
 	set_state = set_state_textmode;
 	WindowResized = WindowResized_Textmode;
 
@@ -1320,16 +1399,20 @@ static void x11_SetTextMode (unsigned char x)
 
 	if (x == Console.CurrentMode)
 	{
-		bzero (Console.VidMem, Console.GraphBytesPerLine * Console.GraphLines);
+		memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
 		return;
 	}
 
-	x11_SetGraphMode (-1);
-
-	destroy_image();
+	if (cachemode != -1)
+	{
+		x11_SetGraphMode (-1);
+		destroy_image();
+	}
 
 	if (x==255)
 	{
+		destroy_image();
+
 		if (window)
 		{
 			vo_showcursor(mDisplay, window);
@@ -1352,10 +1435,13 @@ static void x11_SetTextMode (unsigned char x)
 		return; /* gdb helper */
 	}
 
+#if 0 // We have disabled textmode from being able to change active resolution of physical display - Modern systems do not change resolution away from native the native resolution
 	/* if invalid mode, set it to zero */
 	if (x>7)
+	{
 		x=0;
-#if 0 // We have disabled textmode being able to change resolution of Screen - Modern systems do not change resolution away from native from the monitor
+	}
+
 	if (do_fullscreen)
 	{
 		if (!modelines[modes[x].modeline])
@@ -1389,15 +1475,27 @@ static void x11_SetTextMode (unsigned char x)
 	___push_key (VIRT_KEY_RESIZE);
 
 #else
-	/* WindowResized_Textmode() will override these later when the final result is available */
-	Console.TextHeight = modes[x].chary;
-	Console.TextWidth = modes[x].charx;
-	plScrRowBytes = Console.TextWidth * 2;
-	Console.GraphBytesPerLine = modes[x].windowx;
-	Console.GraphLines = modes[x].windowy;
-	___push_key(VIRT_KEY_RESIZE);
-#endif
+	/* if invalid mode, set it to zero */
+	if (x>=8)
+	{
+		x=8;
+		Console.TextHeight = Textmode_Window_Height / (x11_CurrentFontWanted == _8x16 ? 16 : 8);
+		Console.TextWidth = Textmode_Window_Width / 8;
+		Console.GraphBytesPerLine = Textmode_Window_Width;
+		Console.GraphLines = Textmode_Window_Height;
+	} else {
+		/* WindowResized_Textmode() will override these later when the final result is available */
+		Console.TextHeight = modes[x].chary;
+		Console.TextWidth = modes[x].charx;
+		Console.GraphBytesPerLine = modes[x].windowx;
+		Console.GraphLines = modes[x].windowy;
 
+		x11_CurrentFontWanted = modes[x].bigfont ? _8x16 : _8x8;
+		Textmode_Window_Width = Console.TextWidth * 8;
+		Textmode_Window_Height = Console.TextHeight * (modes[x].bigfont ? 16 : 8);
+	}
+	plScrRowBytes = Console.TextWidth * 2;
+#endif
 	Console.LastTextMode = Console.CurrentMode = x;
 
 	x11_depth=XDefaultDepth(mDisplay, mScreen);
@@ -1408,6 +1506,8 @@ static void x11_SetTextMode (unsigned char x)
 
 	TextModeSetState(x11_CurrentFontWanted /* modes[x].bigfont ? _8x16 : _8x8 */, do_fullscreen);
 
+	___push_key(VIRT_KEY_RESIZE);
+
 	x11_gFlushPal ();
 }
 
@@ -1778,9 +1878,11 @@ static void RefreshScreenGraph(void)
 		}
 	}
 
+#ifdef ENABLE_SHM
 	if (shm_completiontype>=0)
 		XShmPutImage(mDisplay, window, copyGC, image, 0, 0, 0, (Console.GraphLines == 240 ? 20 : 0), Console.GraphBytesPerLine, Console.GraphLines, True);
 	else
+#endif
 		XPutImage(mDisplay, window, copyGC, image, 0, 0, 0, (Console.GraphLines == 240 ? 20 : 0), Console.GraphBytesPerLine, Console.GraphLines);
 
 	if (Console.CurrentFont == _8x8)
@@ -1810,7 +1912,7 @@ static void RefreshScreenText (void)
 
 static int ekbhit_x11dummy (void)
 {
-	if (Console.CurrentMode < 8)
+	if (Console.CurrentMode < 9)
 	{
 		RefreshScreenText();
 	} else {
@@ -2089,7 +2191,7 @@ static void x11_DisplaySetupTextMode (void)
 	{
 		uint16_t c;
 
-		bzero (Console.VidMem, Console.GraphBytesPerLine * Console.GraphLines);
+		memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
 
 		make_title("x11-driver setup", 0);
 		swtext_displaystr_cp437 (1, 0, 0x07, "1:  font-size:", 14);
@@ -2116,7 +2218,7 @@ static void x11_DisplaySetupTextMode (void)
 				x11_CurrentFontWanted = (x11_CurrentFontWanted == _8x8)?_8x16 : _8x8;
 				TextModeSetState (x11_CurrentFontWanted, do_fullscreen);
 				x11_CurrentFontWanted = Console.CurrentFont;
-				cfSetProfileInt("x11", "font", Console.CurrentFont, 10);
+				cfSetProfileInt(cfScreenSec, "fontsize", Console.CurrentFont, 10);
 				break;
 			case KEY_EXIT:
 			case KEY_ESC: return;
@@ -2179,7 +2281,10 @@ int x11_init(int use_explicit)
 	if ( (!use_explicit) && (!cfGetProfileBool("x11", "autodetect", 1, 0)) )
 		return -1;
 
-	x11_CurrentFontWanted = cfGetProfileInt("x11", "font", _8x8, 10);
+	x11_CurrentFontWanted = cfGetProfileInt(cfScreenSec, "fontsize", _8x8, 10);
+	Textmode_Window_Width  = saturate (cfGetProfileInt(cfScreenSec, "winwidth",  1280, 10), 640, 65535);
+	Textmode_Window_Height = saturate (cfGetProfileInt(cfScreenSec, "winheight", 1024, 10), 400, 65535);
+
 	switch (x11_CurrentFontWanted)
 	{
 		default:
@@ -2212,7 +2317,7 @@ int x11_init(int use_explicit)
 	Console.Driver = &x11ConsoleDriver;
 	___setup_key(ekbhit_x11dummy, ekbhit_x11dummy); /* filters in more keys */
 
-	x11_SetTextMode (0);
+	x11_SetTextMode (saturate(cfGetProfileInt(cfScreenSec, "screentype", 7, 10), 0, 8));
 
 	Console.TextGUIOverlay = (x11_depth !=8);
 
